import copy
from collections import OrderedDict
from dataclasses import asdict, dataclass, field
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, Union

from evalscope.constants import OutputType

if TYPE_CHECKING:
    from evalscope.api.benchmark import DataAdapter


@dataclass
class BenchmarkMeta:
    """Metadata for a benchmark, including dataset and model configurations."""

    name: str
    """ Unique name of the benchmark."""

    dataset_id: str
    """ Dataset id on modelscope or path to local dataset."""

    data_adapter: Optional[Type['DataAdapter']] = None
    """ Data adapter class for the benchmark."""

    output_types: List[str] = field(default_factory=lambda: [OutputType.GENERATION])
    """ List of output types supported by the benchmark."""

    subset_list: List[str] = field(default_factory=lambda: ['default'])
    """ List of subsets available for the benchmark."""

    default_subset: str = 'default'
    """ Default subset to use for the benchmark."""

    few_shot_num: int = 0
    """ Number of few-shot examples to use."""

    few_shot_random: bool = False
    """ Whether to use random few-shot examples."""

    train_split: Optional[str] = None
    """ Training split to use for the benchmark."""

    eval_split: Optional[str] = None
    """ Evaluation split to use for the benchmark."""

    prompt_template: Optional[str] = None
    """ Prompt template to use for the benchmark."""

    few_shot_prompt_template: Optional[str] = None
    """ Few-shot prompt template to use for the benchmark."""

    system_prompt: Optional[str] = None
    """ System prompt to use for the benchmark."""

    query_template: Optional[str] = None
    """ Query template to use for the benchmark."""

    pretty_name: Optional[str] = None
    """ Human-readable name for the benchmark."""

    description: Optional[str] = None
    """ Description of the benchmark."""

    tags: List[str] = field(default_factory=list)
    """ Tags associated with the benchmark."""

    filters: Optional[OrderedDict] = None
    """ Filters to apply to the dataset on model output."""

    metric_list: List[Union[str, Dict[str, Any]]] = field(default_factory=list)
    """ List of metrics to evaluate the benchmark."""

    aggregation: str = 'mean'
    """ Aggregation function for the metrics. Default is 'mean'. Can be 'mean', 'pass@<k>' or a custom function name."""

    shuffle: bool = False
    """Whether to shuffle the dataset before evaluation."""

    shuffle_choices: bool = False
    """Whether to shuffle the choices in multiple-choice datasets."""

    review_timeout: Optional[float] = None
    """ Timeout for review in seconds."""

    extra_params: Dict = field(default_factory=dict)
    """ Additional parameters for the benchmark."""

    def __post_init__(self):
        """Validate fields after initialization."""
        if self.few_shot_num < 0:
            raise ValueError('few_shot_num must be >= 0')

    def _update(self, args: dict):
        """Update instance with provided arguments, maintaining backward compatibility."""
        args = copy.deepcopy(args)

        if args.get('local_path'):
            self.dataset_id = args['local_path']
            del args['local_path']

        if args.get('filters'):
            if self.filters is None:
                self.filters = OrderedDict()
            new_filters = OrderedDict(args['filters'])
            # insert filters at the beginning
            self.filters = OrderedDict(list(new_filters.items()) + list(self.filters.items()))
            del args['filters']
        # Update fields with validation
        for key, value in args.items():
            if hasattr(self, key):
                setattr(self, key, value)  # Validate few_shot_num if it's being updated
                if key == 'few_shot_num' and value < 0:
                    raise ValueError('few_shot_num must be >= 0')

    def to_dict(self) -> dict:
        """Convert to dictionary, maintaining backward compatibility."""
        return asdict(self)

    def to_string_dict(self) -> dict:
        """Convert to string dictionary, excluding data_adapter."""
        cur_dict = copy.deepcopy(asdict(self))
        if 'data_adapter' in cur_dict:
            del cur_dict['data_adapter']
        return cur_dict



from typing import TYPE_CHECKING, Any, Dict, List, Optional

from evalscope.api.evaluator import TaskState
from evalscope.api.metric import Score
from evalscope.constants import JudgeStrategy
from evalscope.metrics import LLMJudge
from evalscope.utils.logger import get_logger

if TYPE_CHECKING:
    from evalscope.config import TaskConfig

logger = get_logger()


class LLMJudgeMixin:
    """
    Mixin class for LLM Judge functionality.
    """

    def __init__(self, task_config: 'TaskConfig'):
        self._task_config = task_config
        self._use_llm_judge = False
        """Whether to use LLM as a judge"""

        self._llm_judge: Optional[LLMJudge] = None

        super().__init__(task_config=task_config)

    @property
    def llm_judge(self) -> Optional[LLMJudge]:
        """Get LLM judge instance with lazy initialization."""
        if self._llm_judge is None and self.use_llm_judge:
            self._llm_judge = self.init_llm_judge()
        return self._llm_judge

    @llm_judge.setter
    def llm_judge(self, value: Optional[LLMJudge]):
        """Set LLM judge instance."""
        self._llm_judge = value

    @property
    def judge_strategy(self) -> str:
        """Get the judge strategy from the task configuration."""
        return self._task_config.judge_strategy

    @property
    def use_llm_judge(self) -> bool:
        """Check if LLM judge is enabled."""
        if self.judge_strategy == JudgeStrategy.RULE:
            return False
        elif self.judge_strategy == JudgeStrategy.LLM:
            return True
        elif self.judge_strategy == JudgeStrategy.LLM_RECALL:
            return True
        elif self.judge_strategy == JudgeStrategy.AUTO:
            return self._use_llm_judge
        else:
            logger.warning(f'Unknown judge strategy: {self.judge_strategy}. Defaulting to False.')
            return False

    def init_llm_judge(self) -> Optional[LLMJudge]:
        """
        Initialize the LLM judge for the benchmark.

        Returns:
            Optional[LLMJudge]: The initialized LLM judge instance or None
        """

        if self.judge_strategy == JudgeStrategy.RULE:
            return None
        else:
            return LLMJudge(**self._task_config.judge_model_args)

    def maybe_llm_match_score(
        self,
        original_prediction: str,
        filtered_prediction: str,
        reference: str,
        task_state: TaskState,
        rule_based_score: Optional[Score] = None,
    ) -> Score:
        """
        Compute the match score between the original and filtered predictions against the reference.

        Args:
            original_prediction: The original prediction output from the model.
            filtered_prediction: The filtered prediction output from the model.
            reference: The ground truth reference output.
            task_state: The current task state.
            original_score: Optional original score to be used for comparison.

        Returns:
            Score: The computed match score.
        """
        # If LLM judge is not used, return the rule-based score directly
        if not self.use_llm_judge:
            return rule_based_score

        # For LLM_RECALL, if rule-based score is already perfect, skip LLM judge
        if float(rule_based_score.main_value) > 0.99:
            return rule_based_score

        # Compute LLM judge score
        llm_score = self.llm_match_score(
            original_prediction=original_prediction,
            filtered_prediction=filtered_prediction,
            reference=reference,
            task_state=task_state,
        )

        # For LLM RECALL, merge the scores
        return self._merge_scores(rule_based_score, llm_score)

    def llm_match_score(
        self,
        original_prediction: str,
        filtered_prediction: str,
        reference: str,
        task_state: TaskState,
    ) -> Score:
        """Compute the LLM match score.

        Args:
            original_prediction (str): The original prediction output from the model.
            filtered_prediction (str): The filtered prediction output from the model.
            reference (str): The ground truth reference output.
            task_state (TaskState): The current task state.

        Returns:
            Score: The computed match score.
        """
        score = Score(
            extracted_prediction=filtered_prediction,
            prediction=original_prediction,
        )

        question = task_state.input_text

        # Request judge and obtain score
        prompt = self.llm_judge.build_prompt(pred=original_prediction, gold=reference, question=question)
        judge_response = self.llm_judge.judge(prompt)
        judge_score = self.llm_judge.get_score(judge_response)

        score.value = {'acc': judge_score}
        score.explanation = f'LLM judge: {judge_response}'
        score.metadata = {
            'source': 'llm_judge',
            'judge_strategy': self.judge_strategy,
            'model': self.llm_judge.model_id
        }

        return score

    def _merge_scores(self, rule_based_score: Score, llm_score: Score) -> Score:
        """
        Merge rule-based score with LLM judge score for LLM_RECALL strategy.

        Args:
            rule_based_score: The original rule-based score
            llm_score: The LLM judge score

        Returns:
            Score: The merged score
        """
        # Update the main value with LLM judge result
        rule_based_score.main_value = llm_score.main_value
        rule_based_score.explanation = llm_score.explanation
        rule_based_score.metadata = llm_score.metadata

        return rule_based_score



def get_entity_mappings():
    entity_type_map = {
        'ALBUM': 'album',
        'AWARD': 'award',
        'BAND': 'band',
        'COUNTRY': 'country',
        'EVENT': 'event',
        'LOCATION': 'location',
        'MISC': 'misc',
        'MUSICALARTIST': 'musical_artist',
        'MUSICALINSTRUMENT': 'musical_instrument',
        'MUSICGENRE': 'music_genre',
        'ORGANISATION': 'organisation',
        'PERSON': 'person',
        'SONG': 'song'
    }
    entity_descriptions = {
        'ALBUM': ('The title of a music album (e.g., "Abbey Road", "Thriller", "Lemonade").'),
        'AWARD': ('A music award or prize (e.g., "Grammy Award", "MTV Music Award").'),
        'BAND': ('The name of a musical group or band (e.g., "The Beatles", "Queen", "BTS").'),
        'COUNTRY': ('A country relevant to the music context (e.g., "USA", "UK", "South Korea").'),
        'EVENT': ('A music festival, concert tour, or event (e.g., "Glastonbury Festival", '
                  '"Woodstock").'),
        'LOCATION':
        ('A venue, studio, or place relevant to music (e.g., "Madison Square Garden", '
         '"Abbey Road Studios").'),
        'MISC': ('Miscellaneous music-related terms (e.g., "synthesizer", "major key", '
                 '"a cappella").'),
        'MUSICALARTIST': ('A solo musician or singer (e.g., "Michael Jackson", "Taylor Swift", '
                          '"Ed Sheeran").'),
        'MUSICALINSTRUMENT': ('A musical instrument (e.g., "guitar", "piano", "violin").'),
        'MUSICGENRE': ('A genre or style of music (e.g., "Rock", "Pop", "Jazz", "K-Pop").'),
        'ORGANISATION': ('A record label or music organization (e.g., "Capitol Records", "Sony Music").'),
        'PERSON':
        ('A person related to music who is not a primary artist (e.g., a producer, '
         'a songwriter, "John Lennon").'),
        'SONG': ('The title of a song (e.g., "Bohemian Rhapsody", "Hey Jude", "Dynamite").')
    }
    return entity_type_map, entity_descriptions



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import re
from typing import Any, Dict

from evalscope.api.benchmark import BenchmarkMeta, DefaultDataAdapter
from evalscope.api.dataset import Sample
from evalscope.api.evaluator import TaskState
from evalscope.api.registry import register_benchmark
from evalscope.constants import Tags
from evalscope.utils.logger import get_logger

logger = get_logger()

# BBH multiple choice subset list
MULTIPLE_CHOICE = 'multiple_choice'
MULTIPLE_CHOICE_LIST = [
    'temporal_sequences',
    'disambiguation_qa',
    'date_understanding',
    'tracking_shuffled_objects_three_objects',
    'penguins_in_a_table',
    'geometric_shapes',
    'snarks',
    'ruin_names',
    'tracking_shuffled_objects_seven_objects',
    'tracking_shuffled_objects_five_objects',
    'logical_deduction_three_objects',
    'hyperbaton',
    'logical_deduction_five_objects',
    'logical_deduction_seven_objects',
    'movie_recommendation',
    'salient_translation_error_detection',
    'reasoning_about_colored_objects',
]

# The free form subset list of BBH dataset
FREE_FORM = 'free_form'
FREE_FORM_LIST = [
    'multistep_arithmetic_two',
    'navigate',
    'dyck_languages',
    'word_sorting',
    'sports_understanding',
    'boolean_expressions',
    'object_counting',
    'formal_fallacies',
    'causal_judgement',
    'web_of_lies',
]

# BBH sub-task type
TASK_TYPE = 'task_type'
SUBSET_LIST = MULTIPLE_CHOICE_LIST + FREE_FORM_LIST

PROMPT_TEMPLATE = """
Q: {question}
A: Let's think step by step. Put your final answer in the format of "So the answer is $ANSWER" (without quotes and markdown) where $ANSWER is the answer to the problem.
""".lstrip()  # noqa: E501

FEWSHOT_TEMPLATE = """
{fewshot}

""".lstrip() + PROMPT_TEMPLATE


@register_benchmark(
    BenchmarkMeta(
        name='bbh',
        pretty_name='BBH',
        dataset_id='evalscope/bbh',
        tags=[Tags.REASONING],
        description=
        'The BBH (Big Bench Hard) benchmark is a collection of challenging tasks designed to evaluate the reasoning capabilities of AI models. It includes both free-form and multiple-choice tasks, covering a wide range of reasoning skills.',  # noqa: E501
        subset_list=SUBSET_LIST,
        few_shot_num=3,
        train_split=None,
        eval_split='test',
        metric_list=['acc'],
        prompt_template=PROMPT_TEMPLATE,
        few_shot_prompt_template=FEWSHOT_TEMPLATE,
    )
)
class BBHAdapter(DefaultDataAdapter):
    """
    Adapter for BBH free-form and multiple-choices sub-tasks.
    """

    def __init__(self, **kwargs):
        few_shot_num = kwargs.get('few_shot_num', 3)

        if few_shot_num != 3 and few_shot_num != 0:
            logger.error(
                f'BBH uses 3-shot examples with CoT or 0-shot by system, but got {few_shot_num}. '
                f'Use 3-shot by default.'
            )
            kwargs['few_shot_num'] = 3

        super().__init__(**kwargs)

    def record_to_sample(self, record: Dict[str, Any]) -> Sample:
        input = record['input']
        target = record['target'].replace('(', '').replace(')', '').strip()  # Clean up the target answer

        # Determine task type based on subset name
        task_type = None
        subset_name = self.current_subset_name
        if subset_name in MULTIPLE_CHOICE_LIST:
            task_type = MULTIPLE_CHOICE
        elif subset_name in FREE_FORM_LIST:
            task_type = FREE_FORM

        metadata = {TASK_TYPE: task_type}

        return Sample(input=input, target=target, metadata=metadata, subset_key=subset_name)

    def format_fewshot_template(self, fewshot: str, sample: Sample) -> str:
        # Load CoT prompts from file for BBH
        subset_name = sample.subset_key
        if subset_name:
            cot_file_path = os.path.join(os.path.dirname(__file__), 'cot_prompts', f'{subset_name}.txt')
            if os.path.exists(cot_file_path):
                with open(cot_file_path, 'r', encoding='utf-8') as f:
                    fewshot = f.read().strip()
        return self.few_shot_prompt_template.format(
            fewshot=fewshot,
            question=sample.input,
        )

    def extract_answer(self, prediction: str, task_state: TaskState):
        task_type = task_state.metadata.get(TASK_TYPE)

        if task_type == MULTIPLE_CHOICE:
            return self._extract_mc_answer(prediction)
        elif task_type == FREE_FORM:
            return self._extract_ff_answer(prediction)
        else:
            return prediction.strip()

    @classmethod
    def _extract_mc_answer(cls, ans: str) -> str:
        """
        Extract normalized answer for BBH multiple-choice tasks.
        Handles formats like:
        - "answer is (A)"
        - "The answer is A."
        - Extra text after answer.
        Always uses the *last* occurrence of "answer is".
        """
        ans = ans.strip()

        parts = ans.split('So the answer is ')
        if len(parts) > 1:
            ans = parts[-1].strip()
        ans = ans.split('\n')[0].strip()

        # Remove trailing period
        if ans.endswith('.'):
            ans = ans[:-1].strip()

        # Capture uppercase letter inside parentheses (A) (B) ...
        match = re.search(r'\(([A-Z])\)', ans)
        if match:
            return match.group(1)

        # Capture single uppercase letter
        match = re.search(r'\b([A-Z])\b', ans)
        if match:
            return match.group(1)

        return ans

    @classmethod
    def _extract_ff_answer(cls, ans: str):
        """
        Extract the normalized answer for BBH free-form tasks.
        Handles patterns like:
        - "answer is XXX."
        - "The answer is **valid**."
        - Extra trailing dots / line breaks.
        - Bold-marked answers (**xxx**).
        Always uses the *last* occurrence of "answer is".
        """
        ans = ans.strip()

        parts = ans.split('So the answer is ')
        if len(parts) > 1:
            ans = parts[-1].strip()
        ans = ans.split('\n')[0].strip()

        # Remove trailing period
        if ans.endswith('.'):
            ans = ans[:-1].strip()

        # If answer is in bold (**xxx**), prefer the content inside
        match = re.search(r'\*\*(.*?)\*\*', ans)
        if match:
            ans = match.group(1).strip()

        return ans



import re
from typing import List

from evalscope.api.filter import Filter
from evalscope.api.registry import register_filter


@register_filter('regex')
class RegexFilter(Filter):
    """A filter that extracts values from text using regex pattern matching.

    This filter applies a regex pattern to each model response and extracts matched values.
    If no match is found, returns a fallback value. Useful for extracting structured data
    (like numbers) from unstructured model outputs.
    """

    def __init__(
        self,
        regex_pattern: str = r'#### (\-?[0-9\.\,]+)',
        group_select: int = 0,
        fallback: str = '[invalid]',
    ) -> None:
        """
        pass a string `regex` to run `re.compile(r"regex")` on.
        `fallback` defines the output returned if no matches for the regex are located.
        """
        self.regex_pattern = regex_pattern
        self.regex = re.compile(regex_pattern)
        self.group_select = group_select
        self.fallback = fallback

    def apply(self, instance: List[str]) -> List[str]:
        """Apply regex pattern to each string in the instance list."""
        filtered = []
        for resp in instance:
            match = self.regex.findall(resp)
            if match:
                match = match[self.group_select]
                if isinstance(match, tuple):
                    match = [m for m in match if m]
                    if match:
                        match = match[0]
                    else:
                        match = self.fallback
                match = match.strip()
            else:
                match = self.fallback
            filtered.append(match)
        return filtered


@register_filter('regex_pos')
class POSFilter(Filter):
    """ """

    def __init__(
        self,
        regex_pattern: str = r"\['(.*?)'\]",
        group_select=0,
        fallback=None,
    ) -> None:
        """
        pass a string `regex` to run `re.compile(r"regex")` on.
        `fallback` defines the output returned if no matches for the regex are located.
        """
        if fallback is None:
            fallback = ['invalid']
        self.regex_pattern = regex_pattern
        self.regex = re.compile(regex_pattern)
        self.group_select = group_select
        self.fallback = fallback

    def apply(self, instance: List[str]) -> List[str]:
        """Extract POS tags from each string in the instance list."""

        def extract_tagged_tokens(text):
            # Extract tagged tokens list from text input using regex
            tokens = re.findall(r"\('([^']*)', '([^']*)'\)", text)
            return [(token, pos) for token, pos in tokens]

        def extract_pos_tags(result):
            pos_tags = []
            if isinstance(result, str):
                result = extract_tagged_tokens(result)
            pos_tags.extend(pos for _, pos in result)
            return pos_tags if pos_tags else self.fallback

        filtered = []
        for resp in instance:
            match = extract_pos_tags(resp)
            filtered.append(str(match))
        return filtered


@register_filter('remove_whitespace')
class WhitespaceFilter(Filter):
    """Filters out leading whitespace from responses."""

    def apply(self, instance: List[str]) -> List[str]:
        """Remove leading whitespace from each string in the instance list."""
        filtered_resp = []
        for resp in instance:
            resp = resp.lstrip()
            filtered_resp.append(resp)
        return filtered_resp


@register_filter('remove_until')
class RemoveUntilFilter(Filter):
    """Filters out all text until a specified delimiter is found."""

    def __init__(self, delimiter: str) -> None:
        self.delimiter = delimiter

    def apply(self, instance: List[str]) -> List[str]:
        """Remove all text until the delimiter from each string in the instance list."""
        filtered_resp = []
        for resp in instance:
            resp = resp.split(self.delimiter, 1)[-1]
            filtered_resp.append(resp)
        return filtered_resp


@register_filter('extract')
class ExtractFilter(RegexFilter):
    ...



# flake8: noqa
import copy
import jieba
import Levenshtein
import pandas as pd
import re
from apted import APTED, Config
from apted.helpers import Tree
from collections import defaultdict, deque
from lxml import etree, html
from nltk.translate.bleu_score import corpus_bleu
from nltk.translate.meteor_score import meteor_score
from tabulate import tabulate
from tqdm import tqdm

from evalscope.utils import get_logger
from .utils import normalized_table

logger = get_logger()


def show_result(results):
    for metric_name in results.keys():
        score_table = [[k, v] for k, v in results[metric_name].items()]
        logger.info(f'\n{metric_name}:\n' + tabulate(score_table) + '\n' + '=' * 100)


def sort_nested_dict(d):
    # If it's a dictionary, recursively sort it
    if isinstance(d, dict):
        # Sort the current dictionary
        sorted_dict = {k: sort_nested_dict(v) for k, v in sorted(d.items())}
        return sorted_dict
    # If not a dictionary, return directly
    return d


def get_full_labels_results(samples: dict):
    if not samples:
        return {}
    label_group_dict = defaultdict(lambda: defaultdict(list))
    for sample in samples:
        label_list = []
        if not sample.get('gt_attribute'):
            continue
        for anno in sample['gt_attribute']:
            for k, v in anno.items():
                label_list.append(k + ': ' + str(v))
        for label_name in list(
            set(label_list)
        ):  # Currently if there are merged cases, calculate based on the set of all labels involved after merging
            for metric, score in sample['metric'].items():
                label_group_dict[label_name][metric].append(score)

    logger.info('----Anno Attribute---------------')
    result = {}
    result['sample_count'] = {}
    for attribute in label_group_dict.keys():
        for metric, scores in label_group_dict[attribute].items():
            mean_score = sum(scores) / len(scores)
            if not result.get(metric):
                result[metric] = {}
            result[metric][attribute] = mean_score
            result['sample_count'][attribute] = len(scores)
    result = sort_nested_dict(result)
    show_result(result)
    return result


def get_page_split(samples, page_info):  # Page level metric
    if not page_info:
        return {}
    result_list = defaultdict(list)

    for sample in samples:
        img_name = sample['img_id'] if sample['img_id'].endswith('.jpg') else '_'.join(sample['img_id'].split('_')[:-1])
        page_info_s = page_info[img_name]
        if not sample.get('metric'):
            continue
        for metric, score in sample['metric'].items():
            gt = sample['norm_gt'] if sample.get('norm_gt') else sample['gt']
            pred = sample['norm_pred'] if sample.get('norm_pred') else sample['pred']
            result_list[metric].append({
                'image_name': img_name,
                'metric': metric,
                'attribute': 'ALL',
                'score': score,
                'upper_len': max(len(gt), len(pred))
            })
            for k, v in page_info_s.items():
                if isinstance(v, list):  # special issue
                    for special_issue in v:
                        if 'table' not in special_issue:  # Table-related special fields have duplicates
                            result_list[metric].append({
                                'image_name': img_name,
                                'metric': metric,
                                'attribute': special_issue,
                                'score': score,
                                'upper_len': max(len(gt), len(pred))
                            })
                else:
                    result_list[metric].append({
                        'image_name': img_name,
                        'metric': metric,
                        'attribute': k + ': ' + str(v),
                        'score': score,
                        'upper_len': max(len(gt), len(pred))
                    })

    # Page level logic, accumulation is only done within pages, and mean operation is performed between pages
    result = {}
    if result_list.get('Edit_dist'):
        df = pd.DataFrame(result_list['Edit_dist'])
        up_total_avg = df.groupby(
            ['image_name', 'attribute']
        ).apply(lambda x: (x['score'] * x['upper_len']).sum() / x['upper_len'].sum(),
                include_groups=False).groupby('attribute').mean(
                )  # At page level, accumulate edits, denominator is sum of max(gt, pred) from each sample
        result['Edit_dist'] = up_total_avg.to_dict()
    for metric in result_list.keys():
        if metric == 'Edit_dist':
            continue
        df = pd.DataFrame(result_list[metric])
        page_avg = df.groupby(['image_name', 'attribute']).apply(lambda x: x['score'].mean(),
                                                                 include_groups=False).groupby('attribute').mean()
        result[metric] = page_avg.to_dict()

    result = sort_nested_dict(result)
    # print('----Page Attribute---------------')
    show_result(result)
    return result


def get_groups(samples, group_info):
    group_samples = defaultdict(list)
    for sample in samples:
        group_samples['all'].append(sample)
        for group in group_info:
            select_flag = True
            for k, v in group.items():
                for gt_attribute in sample['gt_attribute'
                                           ]:  # gt_attribute is a list containing all merged gt attributes
                    if not gt_attribute:  # if no GT attributes, don't include in calculation
                        select_flag = False
                    elif gt_attribute[k] != v:  # if any gt attribute doesn't meet criteria, don't select
                        select_flag = False
            if select_flag:
                group_samples[str(group)].append(sample)
    return group_samples


class Registry:

    def __init__(self):
        self._registry = {}

    def register(self, name):

        def decorator(item):
            if name in self._registry:
                raise ValueError(f'Item {name} already registered.')
            self._registry[name] = item
            return item

        return decorator

    def get(self, name):
        if name not in self._registry:
            raise ValueError(f'Item {name} not found in registry.')
        return self._registry[name]

    def list_items(self):
        return list(self._registry.keys())


METRIC_REGISTRY = Registry()


@METRIC_REGISTRY.register('TEDS')
class call_TEDS():

    def __init__(self, samples):
        self.samples = samples

    def evaluate(self, group_info=[], save_name='default'):
        teds = TEDS(structure_only=False)
        teds_structure_only = TEDS(structure_only=True)

        group_scores = defaultdict(list)
        group_scores_structure_only = defaultdict(list)

        samples = self.samples
        for sample in samples:
            gt = sample['norm_gt'] if sample.get('norm_gt') else sample['gt']
            pred = sample['norm_pred'] if sample.get('norm_pred') else sample['pred']

            score = teds.evaluate(pred, gt)
            score_structure_only = teds_structure_only.evaluate(pred, gt)
            # print('TEDS score:', score)
            group_scores['all'].append(score)
            group_scores_structure_only['all'].append(score_structure_only)

            if not sample.get('metric'):
                sample['metric'] = {}
            sample['metric']['TEDS'] = score
            sample['metric']['TEDS_structure_only'] = score_structure_only

            for group in group_info:
                select_flag = True
                for k, v in group.items():
                    for gt_attribute in sample['gt_attribute'
                                               ]:  # gt_attribute is a list containing all merged gt attributes
                        if not gt_attribute:  # if no GT attributes, don't include in calculation
                            select_flag = False
                        elif gt_attribute[k] != v:  # if any gt attribute doesn't meet criteria, don't select
                            select_flag = False
                if select_flag:
                    group_scores[str(group)].append(score)

        result = {}
        for group_name, scores in group_scores.items():
            if len(scores) > 0:
                result[group_name] = sum(scores) / len(scores)  # average of normalized scores at sample level
            else:
                result[group_name] = 'NaN'
                logger.warning(f'Empty matched samples for {group_name}.')

        structure_only_result = {}
        for group_name, scores in group_scores_structure_only.items():
            if len(scores) > 0:
                structure_only_result[group_name] = sum(scores) / len(
                    scores
                )  # average of normalized scores at sample level
            else:
                structure_only_result[group_name] = 'NaN'
                logger.warning(f'Empty matched samples for {group_name}.')

        return samples, {'TEDS': result, 'TEDS_structure_only': structure_only_result}


def tokenize(text) -> list[str]:
    """Tokenizes text, handling Chinese and non-Chinese strings appropriately."""

    def contain_chinese_string(text):
        chinese_pattern = re.compile(r'[\u4e00-\u9fa5]')
        return bool(chinese_pattern.search(text))

    if contain_chinese_string(text):
        res = jieba.lcut(text)
    else:
        res = text.split()
    return res


@METRIC_REGISTRY.register('BLEU')
class call_BLEU():

    def __init__(self, samples):
        self.samples = samples

    def evaluate(self, group_info=[], save_name='default'):
        group_samples = get_groups(self.samples, group_info)
        result = {}

        for group_name, samples in group_samples.items():
            predictions, references = [], []
            for sample in samples:
                gt = sample['norm_gt'] if sample.get('norm_gt') else sample['gt']
                pred = sample['norm_pred'] if sample.get('norm_pred') else sample['pred']
                predictions.append(tokenize(pred))
                references.append([tokenize(gt)])

            if not predictions or not any(predictions) or not references or not any(references):
                bleu_score = 0
            else:
                try:
                    bleu_score = corpus_bleu(references, predictions)
                except ZeroDivisionError:
                    bleu_score = 0

            result[group_name] = bleu_score

        return self.samples, {'BLEU': result}


@METRIC_REGISTRY.register('METEOR')
class call_METEOR():

    def __init__(self, samples):
        self.samples = samples

    def evaluate(self, group_info=[], save_name='default'):
        group_samples = get_groups(self.samples, group_info)
        result = {}
        for group_name, samples in group_samples.items():
            predictions, references = [], []
            for sample in samples:
                gt = sample['norm_gt'] if sample.get('norm_gt') else sample['gt']
                pred = sample['norm_pred'] if sample.get('norm_pred') else sample['pred']
                predictions.append(tokenize(pred))
                references.append(tokenize(gt))
            # Calculate METEOR score
            if not predictions or not references:
                meteor_results = 0
            else:
                try:
                    total_score = 0
                    for ref, pred in zip(references, predictions):
                        score = meteor_score([ref], pred)
                        total_score += score
                    meteor_results = total_score / len(references)
                except Exception as e:
                    logger.error(f'METEOR calculation error: {e}')
                    meteor_results = 0
            result[group_name] = meteor_results

        return self.samples, {'METEOR': result}


@METRIC_REGISTRY.register('Edit_dist')
class call_Edit_dist():

    def __init__(self, samples):
        self.samples = samples

    def evaluate(self, group_info=[], save_name='default'):
        samples = self.samples
        for sample in samples:
            img_name = sample['img_id'] if sample['img_id'].endswith('.jpg') else '_'.join(
                sample['img_id'].split('_')[:-1]
            )
            sample['image_name'] = img_name
            gt = sample['norm_gt'] if sample.get('norm_gt') else sample['gt']
            pred = sample['norm_pred'] if sample.get('norm_pred') else sample['pred']
            upper_len = max(len(pred), len(gt))
            sample['upper_len'] = upper_len
            if len(pred) > 0 or len(gt) > 0:
                edit_dist = Levenshtein.distance(pred, gt)
                if not sample.get('metric'):
                    sample['metric'] = {}
                sample['metric']['Edit_dist'] = edit_dist / upper_len
                sample['Edit_num'] = edit_dist

        if isinstance(samples, list):
            saved_samples = samples
        else:
            saved_samples = samples.samples

        if not saved_samples:
            return self.samples, {'Edit_dist': {'ALL_page_avg': 'NaN'}}

        df = pd.DataFrame(saved_samples)
        up_total_avg = df.groupby('image_name').apply(
            lambda x: x['Edit_num'].sum() / x['upper_len'].sum(), include_groups=False
        )  # page level, sum of edits divided by sum of max(gt,pred) lengths for each sample

        return samples, {'Edit_dist': {'ALL_page_avg': up_total_avg.mean()}}


@METRIC_REGISTRY.register('CDM')
class call_CDM():

    def __init__(self, samples):
        self.samples = samples

    def evaluate(self, group_info=[], save_name='default'):
        if isinstance(self.samples, list):
            cdm_samples = copy.deepcopy(self.samples)
        else:
            cdm_samples = copy.deepcopy(self.samples.samples)
        for idx, sample in enumerate(cdm_samples):
            sample['img_name'] = sample['img_id']
            sample['img_id'] = str(idx)
            sample['gt'] = sample['gt'].lstrip('$$').rstrip('$$').strip()
            sample['pred'] = sample['pred'].split('```latex')[-1].split('```')[0]
            sample['pred'] = sample['pred'].lstrip('$$').rstrip('$$').strip()

        return self.samples, False


class TEDS(object):
    ''' Tree Edit Distance basead Similarity
    '''

    def __init__(self, structure_only=False, n_jobs=1, ignore_nodes=None):
        assert isinstance(n_jobs, int) and (n_jobs >= 1), 'n_jobs must be an integer greather than 1'
        self.structure_only = structure_only
        self.n_jobs = n_jobs
        self.ignore_nodes = ignore_nodes
        self.__tokens__ = []

    def tokenize(self, node):
        ''' Tokenizes table cells
        '''
        self.__tokens__.append('<%s>' % node.tag)
        if node.text is not None:
            self.__tokens__ += list(node.text)
        for n in node.getchildren():
            self.tokenize(n)
        if node.tag != 'unk':
            self.__tokens__.append('</%s>' % node.tag)
        if node.tag != 'td' and node.tail is not None:
            self.__tokens__ += list(node.tail)

    def load_html_tree(self, node, parent=None):
        ''' Converts HTML tree to the format required by apted
        '''
        global __tokens__
        if node.tag == 'td':
            if self.structure_only:
                cell = []
            else:
                self.__tokens__ = []
                self.tokenize(node)
                cell = self.__tokens__[1:-1].copy()
            new_node = TableTree(
                node.tag, int(node.attrib.get('colspan', '1')), int(node.attrib.get('rowspan', '1')), cell, *deque()
            )
        else:
            new_node = TableTree(node.tag, None, None, None, *deque())
        if parent is not None:
            parent.children.append(new_node)
        if node.tag != 'td':
            for n in node.getchildren():
                self.load_html_tree(n, new_node)
        if parent is None:
            return new_node

    def evaluate(self, pred, true):
        ''' Computes TEDS score between the prediction and the ground truth of a
            given sample
        '''
        if (not pred) or (not true):
            return 0.0
        parser = html.HTMLParser(remove_comments=True, encoding='utf-8')
        pred = html.fromstring(pred, parser=parser)
        true = html.fromstring(true, parser=parser)
        if pred.xpath('body/table') and true.xpath('body/table'):
            pred = pred.xpath('body/table')[0]
            true = true.xpath('body/table')[0]
            if self.ignore_nodes:
                etree.strip_tags(pred, *self.ignore_nodes)
                etree.strip_tags(true, *self.ignore_nodes)
            n_nodes_pred = len(pred.xpath('.//*'))
            n_nodes_true = len(true.xpath('.//*'))
            n_nodes = max(n_nodes_pred, n_nodes_true)
            tree_pred = self.load_html_tree(pred)
            tree_true = self.load_html_tree(true)
            distance = APTED(tree_pred, tree_true, CustomConfig()).compute_edit_distance()
            return 1.0 - (float(distance) / n_nodes)
        else:
            return 0.0

    def batch_evaluate(self, pred_json, true_json):
        ''' Computes TEDS score between the prediction and the ground truth of
            a batch of samples
            @params pred_json: {'FILENAME': 'HTML CODE', ...}
            @params true_json: {'FILENAME': {'html': 'HTML CODE'}, ...}
            @output: {'FILENAME': 'TEDS SCORE', ...}
        '''
        samples = true_json.keys()
        # if self.n_jobs == 1:
        scores = [self.evaluate(pred_json.get(filename, ''), true_json[filename]['html']) for filename in tqdm(samples)]
        # else:
        #     inputs = [{'pred': pred_json.get(filename, ''), 'true': true_json[filename]['html']} for filename in samples]
        #     scores = parallel_process(inputs, self.evaluate, use_kwargs=True, n_jobs=self.n_jobs, front_num=1)
        scores = dict(zip(samples, scores))
        return scores


class CustomConfig(Config):

    @staticmethod
    def maximum(*sequences):
        """Get maximum possible value
        """
        return max(map(len, sequences))

    def normalized_distance(self, *sequences):
        """Get distance from 0 to 1
        """
        return float(Levenshtein.distance(*sequences)) / self.maximum(*sequences)

    def rename(self, node1, node2):
        """Compares attributes of trees"""
        if (node1.tag != node2.tag) or (node1.colspan != node2.colspan) or (node1.rowspan != node2.rowspan):
            return 1.
        if node1.tag == 'td':
            if node1.content or node2.content:
                return self.normalized_distance(node1.content, node2.content)
        return 0.


class TableTree(Tree):

    def __init__(self, tag, colspan=None, rowspan=None, content=None, *children):
        self.tag = tag
        self.colspan = colspan
        self.rowspan = rowspan
        self.content = content
        self.children = list(children)

    def bracket(self):
        """Show tree using brackets notation"""
        if self.tag == 'td':
            result = '"tag": %s, "colspan": %d, "rowspan": %d, "text": %s' % \
                     (self.tag, self.colspan, self.rowspan, self.content)
        else:
            result = '"tag": %s' % self.tag
        for child in self.children:
            result += child.bracket()
        return '{{{}}}'.format(result)


class recogition_end2end_base_dataset():

    def __init__(self, samples):
        img_id = 0
        for sample in samples:
            if not sample.get('img_id'):
                sample['img_id'] = img_id
            img_id += 1
        self.samples = samples

    def __getitem__(self, idx):
        return self.samples[idx]


class recogition_end2end_table_dataset(recogition_end2end_base_dataset):

    def __init__(self, samples, table_format):
        self.pred_table_format = table_format
        self.samples = self.normalize_data(samples)

    def normalize_data(self, samples):
        img_id = 0
        for sample in samples:
            p = sample['pred']
            r = sample['gt']
            p = normalized_table(p, self.pred_table_format)
            r = normalized_table(r)
            sample['norm_gt'] = r
            sample['norm_pred'] = p
            sample['img_id'] = sample['img_id'] if sample.get('img_id') else img_id
            img_id += 1

        return samples



import os

from evalscope import TaskConfig, run_task

DASHSCOPE_API_KEY = 'sk-723135c241x'

def eval_distill_qwen():
    model_name = 'DeepSeek-R1-Distill-Qwen-7B'
    dataset_name = 'math_500'
    subsets = ['Level 1', 'Level 2', 'Level 3', 'Level 4', 'Level 5']

    task_config = TaskConfig(
        api_url='http://0.0.0.0:8801/v1/chat/completions',
        model=model_name,
        eval_type='service',
        datasets=[dataset_name],
        dataset_args={dataset_name: {'few_shot_num': 0, 'subset_list': subsets}},
        eval_batch_size=32,
        generation_config={
            'max_tokens': 20000,  # avoid exceed max length
            'temperature': 0.6,
            'top_p': 0.95,
            'n': 1,
        },
    )
    run_task(task_config)


def eval_math_qwen():
    model_name = 'Qwen2.5-Math-7B-Instruct'
    dataset_name = 'math_500'
    subsets = ['Level 1', 'Level 2', 'Level 3', 'Level 4', 'Level 5']

    task_config = TaskConfig(
        api_url='http://0.0.0.0:8801/v1/chat/completions',
        model=model_name,
        eval_type='service',
        datasets=[dataset_name],
        dataset_args={dataset_name: {'few_shot_num': 0, 'subset_list': subsets}},
        eval_batch_size=32,
        generation_config={
            'max_tokens': 3000,  # avoid exceed max length
            'temperature': 0.6,
            'top_p': 0.95,
            'n': 3,
        },
    )
    run_task(task_config)

def eval_r1():
    model_name = 'deepseek-r1'
    dataset_name = 'math_500'
    subsets = ['Level 1', 'Level 2', 'Level 3', 'Level 4', 'Level 5']

    task_config = TaskConfig(
        api_url='https://dashscope.aliyuncs.com/compatible-mode/v1/chat/completions',
        api_key=DASHSCOPE_API_KEY,
        model=model_name,
        eval_type='service',
        datasets=[dataset_name],
        dataset_args={dataset_name: {'few_shot_num': 0, 'subset_list': subsets}},
        eval_batch_size=8,
        generation_config={
            'max_tokens': 20000,  # avoid exceed max length
            'temperature': 0.6,
            'top_p': 0.95,
            'n': 1,
        },
        use_cache='./outputs/20250307_000404',
        timeout=36000,
        stream=True
    )
    run_task(task_config)


def eval_distill_32b():
    model_name = 'deepseek-r1-distill-qwen-32b'
    dataset_name = 'math_500'
    subsets = ['Level 1', 'Level 2', 'Level 3', 'Level 4', 'Level 5']

    task_config = TaskConfig(
        api_url='https://dashscope.aliyuncs.com/compatible-mode/v1/chat/completions',
        api_key=DASHSCOPE_API_KEY,
        model=model_name,
        eval_type='service',
        datasets=[dataset_name],
        dataset_args={dataset_name: {'few_shot_num': 0, 'subset_list': subsets}},
        eval_batch_size=5,
        generation_config={
            'max_tokens': 12000,  # avoid exceed max length
            'temperature': 0.6,
            'top_p': 0.95,
            'n': 1,
        },
        use_cache='./outputs/20250306_235951',
        timeout=32000,
        stream=True

    )
    run_task(task_config)

def eval_qwq():
    model_name = 'qwq-32b-preview'
    dataset_name = 'math_500'
    subsets = ['Level 1', 'Level 2', 'Level 3', 'Level 4', 'Level 5']

    task_config = TaskConfig(
        api_url='https://dashscope.aliyuncs.com/compatible-mode/v1/chat/completions',
        api_key=os.environ['DASHSCOPE_API_KEY'],
        model=model_name,
        eval_type='service',
        datasets=[dataset_name],
        dataset_args={dataset_name: {'few_shot_num': 0, 'subset_list': subsets}},
        eval_batch_size=32,
        generation_config={
            'max_tokens': 8000,  # avoid exceed max length
            'temperature': 0.6,
            'top_p': 0.95,
            'n': 1,
        },
        use_cache='./outputs/20250221_105911'
    )
    run_task(task_config)

if __name__ == '__main__':
    # eval_distill_qwen()
    # eval_math_qwen()
    eval_r1()
    # eval_qwq()
    # eval_distill_32b()



import json

from evalscope.metrics import compute_rouge_score_one_sample


def evaluate_rougel(cand_list: list, ref_list: list):
    if len(ref_list) == 0:
        return 0
    rouge_score = compute_rouge_score_one_sample(cand_list, ref_list)
    rougel = rouge_score.get('rouge-l-f', 0)

    return rougel


def evaluate_action_em(cand_list: list, ref_list: list):
    if len(ref_list) == 0:
        return 0
    em = 0
    for cand, ref in zip(cand_list, ref_list):
        em += (1 if cand == ref else 0)
    return em / len(cand_list)


def evaluate_action_input_f1(action_pred: list, action_ref: list, cand_list: list, ref_list: list):
    easy_f1 = []
    hard_f1 = []
    f1 = []
    for i in range(len(action_pred)):
        ref_action = action_ref[i]
        pred_action = action_pred[i]

        ref_input = ref_list[i]
        cand_input = cand_list[i]

        if ref_action != pred_action:
            easy_f1.append(0)
            hard_f1.append(0)
            f1.append(0)
        else:
            try:
                ref_input_json = json.loads(ref_input)
                try:
                    cand_input_json = json.loads(cand_input)
                    half_match = 0
                    full_match = 0
                    if ref_input_json == {}:
                        if cand_input_json == {}:
                            easy_f1.append(1)
                            f1.append(1)
                        else:
                            easy_f1.append(0)
                            f1.append(0)
                    else:
                        for k, v in ref_input_json.items():
                            if k in cand_input_json.keys():
                                if cand_input_json[k] == v:
                                    full_match += 1
                                else:
                                    half_match += 1

                        recall = (0.5 * half_match + full_match) / (len(ref_input_json) + 1e-30)
                        precision = (0.5 * half_match + full_match) / (len(cand_input_json) + 1e-30)
                        hard_f1.append((2 * recall * precision) / (recall + precision))
                        f1.append((2 * recall * precision) / (recall + precision))
                except Exception:
                    # cand_input = cand_input.replace("\n","").replace("\"","")
                    # ref_input = cand_input.replace("\n","").replace("\"","")
                    # rouge = Rouge()
                    # rouge_score = rouge.get_scores(hyps=[cand_input], refs=[ref_input], avg=True)
                    if ref_input_json == {}:
                        easy_f1.append(0)
                    else:
                        hard_f1.append(0)
                    # hard_f1.append(rouge_score["rouge-l"]["f"])
                    # f1.append(rouge_score["rouge-l"]["f"])
                    f1.append(0)
            except Exception:
                pass

    # 检查列表是否为空，如果为空则返回0
    easy_f1_avg = sum(easy_f1) / len(easy_f1) if easy_f1 else 0
    hard_f1_avg = sum(hard_f1) / len(hard_f1) if hard_f1 else 0
    f1_avg = sum(f1) / len(f1) if f1 else 0

    return easy_f1_avg, hard_f1_avg, f1_avg


def parse_action(text):
    action = 'None'
    action_input = '{}'
    if 'Action Input:' in text:
        input_idx = text.rindex('Action Input:')
        action_input = text[input_idx + len('Action Input:'):].strip()
    else:
        action_input = '{}'

    if 'Action:' in text:
        action_idx = text.rindex('Action:')
        action = text[action_idx + len('Action:'):].strip()
        if 'Action Input:' in action:
            input_idx = action.index('Action Input:')
            action = action[:input_idx].strip()
    else:
        action = 'none'
    return action, action_input


def parse_output(text):
    action, action_input = parse_action(text)
    if action == 'Finish':
        try:
            action_input = json.loads(action_input)
            # print(action_input)
            # print(json.dumps(action_input,indent=2))
            return_type = action_input['return_type']
            if return_type == 'give_answer':
                if 'final_answer' in action_input.keys():
                    answer = str(action_input['final_answer'])
                    if answer.strip() in ['', '.', ',']:
                        answer = 'None'
                else:
                    answer = 'None'
                return 'finish', action, action_input, answer
            else:
                return 'give up', None, None, None
        except Exception:
            return 'give up', None, None, None
    else:
        plan = 'call'
        answer = None
        return plan, action, action_input, answer


def calculate_metrics(data):
    """
    Calculate the metrics for the given data.
    """
    plan_ref = []
    plan_pred = []
    hallu_cases = []
    answer_ref = []
    action_ref = []
    action_input_ref = []
    answer_pred = []
    action_pred = []
    action_input_pred = []
    hallu_pred = 0

    reference = data['target']
    prediction = data['predictions']
    ref_plan, ref_action, ref_input, ref_ans = parse_output(reference)
    # ref_plan: call
    # ref_action: spott
    # ref_input: {"is_id": "city center" }
    # ref_ans: None

    pred_plan, pred_action, pred_input, pred_ans = parse_output(prediction)
    if ref_action is not None and ref_action == 'invalid_hallucination_function_name':
        return {}
    if pred_action is not None and ref_action != 'none' and ref_action not in [t['name'] for t in data['tools']]:
        return {}

    if pred_action is not None and pred_action != 'none' and pred_action not in [t['name'] for t in data['tools']]:
        hallu_pred += 1
        hallu_cases.append(data)

    plan_ref.append(ref_plan)
    plan_pred.append(pred_plan)
    if ref_plan == 'give up':
        pass
    elif ref_plan == 'finish':
        answer_ref.append(ref_ans)
        if pred_ans is None:
            answer_pred.append('none')
        else:
            answer_pred.append(pred_ans)
    else:
        action_ref.append(ref_action)
        action_input_ref.append(ref_input)
        if pred_action is None:
            action_pred.append('none')
        else:
            action_pred.append(pred_action)

        if pred_input is None:
            action_input_pred.append('{}')
        else:
            action_input_pred.append(pred_input)

    metric = {}
    rouge = evaluate_rougel(answer_pred, answer_ref)
    plan_em = evaluate_action_em(cand_list=plan_pred, ref_list=plan_ref)
    action_em = evaluate_action_em(cand_list=action_pred, ref_list=action_ref)
    easy_f1, hard_f1, f1 = evaluate_action_input_f1(action_pred, action_ref, action_input_pred, action_input_ref)
    hallu_rate = hallu_pred
    metric['Act.EM'] = action_em
    metric['F1'] = f1
    metric['HalluRate'] = hallu_rate
    metric['plan_em'] = plan_em
    metric['Easy_F1'] = easy_f1
    metric['Hard_F1'] = hard_f1
    metric['Rouge-L'] = rouge
    return metric



# flake8: noqa
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# encoding=utf8
# File: E2E_iou_1_1.py
# Version: 1.1
# Version info: changes for Python 3
# Date: 2019-12-29
# Description: Evaluation script that computes End to End Recognition. For Text Localization it's used Intersection over Union criteria.
# Average Precision is also calcuted when 'CONFIDENCES' parameter is True
# There are 2 modes to determine if a detection is correct or not:
# with Word Spotting: The detected word must coincide (ingnoring case) to a filtered Ground Truth containing only dictionary words (see include_in_dictionary and include_in_dictionary_transcription functions)
# without Word Spotting: words must be equal excluding a set of special characters

import importlib
from collections import namedtuple

from . import rrc_evaluation_funcs_1_1 as rrc_evaluation_funcs


def evaluation_imports():
    """
    evaluation_imports: Dictionary ( key = module name , value = alias  )  with python modules used in the evaluation.
    """
    return {'Polygon': 'plg', 'numpy': 'np'}


def default_evaluation_params():
    """
    default_evaluation_params: Default parameters to use for the validation and evaluation.
    """
    return {
        'IOU_CONSTRAINT': 0.5,
        'AREA_PRECISION_CONSTRAINT': 0.5,
        'WORD_SPOTTING': False,
        'MIN_LENGTH_CARE_WORD': 3,
        'GT_SAMPLE_NAME_2_ID': 'gt_img_([0-9]+).txt',
        'DET_SAMPLE_NAME_2_ID': 'res_img_([0-9]+).txt',
        'LTRB': False,  # LTRB:2points(left,top,right,bottom) or 4 points(x1,y1,x2,y2,x3,y3,x4,y4)
        'CRLF': False,  # Lines are delimited by Windows CRLF format
        'CONFIDENCES': False,  # Detections must include confidence value. AP will be calculated,
        'SPECIAL_CHARACTERS': "!?.:,*\"()·[]/'",
        'ONLY_REMOVE_FIRST_LAST_CHARACTER': True,
    }


def validate_data(gtFilePath, submFilePath, evaluationParams):
    """
    Method validate_data: validates that all files in the results folder are correct (have the correct name contents).
                            Validates also that there are no missing files in the folder.
                            If some error detected, the method raises the error
    """
    gt = rrc_evaluation_funcs.load_zip_file(gtFilePath, evaluationParams['GT_SAMPLE_NAME_2_ID'])

    subm = rrc_evaluation_funcs.load_zip_file(submFilePath, evaluationParams['DET_SAMPLE_NAME_2_ID'], True)

    # Validate format of GroundTruth
    for k in gt:
        rrc_evaluation_funcs.validate_lines_in_file(k, gt[k], evaluationParams['CRLF'], evaluationParams['LTRB'], True)

    # Validate format of results
    for k in subm:
        if (k in gt) == False:
            raise Exception('The sample %s not present in GT' % k)

        rrc_evaluation_funcs.validate_lines_in_file(
            k, subm[k], evaluationParams['CRLF'], evaluationParams['LTRB'], True, evaluationParams['CONFIDENCES']
        )


def evaluate_method(gtFilePath, submFilePath, evaluationParams):
    """
    Method evaluate_method: evaluate method and returns the results
        Results. Dictionary with the following values:
        - method (required)  Global method metrics. Ex: { 'Precision':0.8,'Recall':0.9 }
        - samples (optional) Per sample metrics. Ex: {'sample1' : { 'Precision':0.8,'Recall':0.9 } , 'sample2' : { 'Precision':0.8,'Recall':0.9 }
    """
    for module, alias in evaluation_imports().items():
        globals()[alias] = importlib.import_module(module)

    def polygon_from_points(points, correctOffset=False):
        """
        Returns a Polygon object to use with the Polygon2 class from a list of 8 points: x1,y1,x2,y2,x3,y3,x4,y4
        """

        if correctOffset:  # this will substract 1 from the coordinates that correspond to the xmax and ymax
            points[2] -= 1
            points[4] -= 1
            points[5] -= 1
            points[7] -= 1

        resBoxes = np.empty([1, 8], dtype='int32')
        resBoxes[0, 0] = int(points[0])
        resBoxes[0, 4] = int(points[1])
        resBoxes[0, 1] = int(points[2])
        resBoxes[0, 5] = int(points[3])
        resBoxes[0, 2] = int(points[4])
        resBoxes[0, 6] = int(points[5])
        resBoxes[0, 3] = int(points[6])
        resBoxes[0, 7] = int(points[7])
        pointMat = resBoxes[0].reshape([2, 4]).T
        return plg.Polygon(pointMat)

    def rectangle_to_polygon(rect):
        resBoxes = np.empty([1, 8], dtype='int32')
        resBoxes[0, 0] = int(rect.xmin)
        resBoxes[0, 4] = int(rect.ymax)
        resBoxes[0, 1] = int(rect.xmin)
        resBoxes[0, 5] = int(rect.ymin)
        resBoxes[0, 2] = int(rect.xmax)
        resBoxes[0, 6] = int(rect.ymin)
        resBoxes[0, 3] = int(rect.xmax)
        resBoxes[0, 7] = int(rect.ymax)

        pointMat = resBoxes[0].reshape([2, 4]).T

        return plg.Polygon(pointMat)

    def rectangle_to_points(rect):
        points = [
            int(rect.xmin),
            int(rect.ymax),
            int(rect.xmax),
            int(rect.ymax),
            int(rect.xmax),
            int(rect.ymin),
            int(rect.xmin),
            int(rect.ymin)
        ]
        return points

    def get_union(pD, pG):
        areaA = pD.area()
        areaB = pG.area()
        return areaA + areaB - get_intersection(pD, pG)

    def get_intersection_over_union(pD, pG):
        try:
            return get_intersection(pD, pG) / get_union(pD, pG)
        except:
            return 0

    def get_intersection(pD, pG):
        pInt = pD & pG
        if len(pInt) == 0:
            return 0
        return pInt.area()

    def compute_ap(confList, matchList, numGtCare):
        correct = 0
        AP = 0
        if len(confList) > 0:
            confList = np.array(confList)
            matchList = np.array(matchList)
            sorted_ind = np.argsort(-confList)
            confList = confList[sorted_ind]
            matchList = matchList[sorted_ind]
            for n in range(len(confList)):
                match = matchList[n]
                if match:
                    correct += 1
                    AP += float(correct) / (n + 1)

            if numGtCare > 0:
                AP /= numGtCare

        return AP

    def transcription_match(
        transGt, transDet, specialCharacters="!?.:,*\"()·[]/'", onlyRemoveFirstLastCharacterGT=True
    ):
        if onlyRemoveFirstLastCharacterGT:
            # special characters in GT are allowed only at initial or final position
            if transGt == transDet:
                return True

            if specialCharacters.find(transGt[0]) > -1:
                if transGt[1:] == transDet:
                    return True

            if specialCharacters.find(transGt[-1]) > -1:
                if transGt[0:len(transGt) - 1] == transDet:
                    return True

            if specialCharacters.find(transGt[0]) > -1 and specialCharacters.find(transGt[-1]) > -1:
                if transGt[1:len(transGt) - 1] == transDet:
                    return True
            return False
        else:
            # Special characters are removed from the begining and the end of both Detection and GroundTruth
            while len(transGt) > 0 and specialCharacters.find(transGt[0]) > -1:
                transGt = transGt[1:]

            while len(transDet) > 0 and specialCharacters.find(transDet[0]) > -1:
                transDet = transDet[1:]

            while len(transGt) > 0 and specialCharacters.find(transGt[-1]) > -1:
                transGt = transGt[0:len(transGt) - 1]

            while len(transDet) > 0 and specialCharacters.find(transDet[-1]) > -1:
                transDet = transDet[0:len(transDet) - 1]

            return transGt == transDet

    def include_in_dictionary(transcription):
        """
        Function used in Word Spotting that finds if the Ground Truth transcription meets the rules to enter into the dictionary. If not, the transcription will be cared as don't care
        """
        # special case 's at final
        if transcription[len(transcription) - 2:] == "'s" or transcription[len(transcription) - 2:] == "'S":
            transcription = transcription[0:len(transcription) - 2]

        # hypens at init or final of the word
        transcription = transcription.strip('-')

        specialCharacters = "'!?.:,*\"()·[]/"
        for character in specialCharacters:
            transcription = transcription.replace(character, ' ')

        transcription = transcription.strip()

        if len(transcription) != len(transcription.replace(' ', '')):
            return False

        if len(transcription) < evaluationParams['MIN_LENGTH_CARE_WORD']:
            return False

        notAllowed = '×÷·'

        range1 = [ord('a'), ord('z')]
        range2 = [ord('A'), ord('Z')]
        range3 = [ord('À'), ord('ƿ')]
        range4 = [ord('Ǆ'), ord('ɿ')]
        range5 = [ord('Ά'), ord('Ͽ')]
        range6 = [ord('-'), ord('-')]

        for char in transcription:
            charCode = ord(char)
            if notAllowed.find(char) != -1:
                return False

            valid = ((charCode >= range1[0] and charCode <= range1[1])
                     or (charCode >= range2[0] and charCode <= range2[1])
                     or (charCode >= range3[0] and charCode <= range3[1])
                     or (charCode >= range4[0] and charCode <= range4[1])
                     or (charCode >= range5[0] and charCode <= range5[1])
                     or (charCode >= range6[0] and charCode <= range6[1]))
            if valid == False:
                return False

        return True

    def include_in_dictionary_transcription(transcription):
        """
        Function applied to the Ground Truth transcriptions used in Word Spotting. It removes special characters or terminations
        """
        # special case 's at final
        if transcription[len(transcription) - 2:] == "'s" or transcription[len(transcription) - 2:] == "'S":
            transcription = transcription[0:len(transcription) - 2]

        # hypens at init or final of the word
        transcription = transcription.strip('-')

        specialCharacters = "'!?.:,*\"()·[]/"
        for character in specialCharacters:
            transcription = transcription.replace(character, ' ')

        transcription = transcription.strip()

        return transcription

    perSampleMetrics = {}

    matchedSum = 0

    Rectangle = namedtuple('Rectangle', 'xmin ymin xmax ymax')

    gt = rrc_evaluation_funcs.load_zip_file(gtFilePath, evaluationParams['GT_SAMPLE_NAME_2_ID'])
    subm = rrc_evaluation_funcs.load_zip_file(submFilePath, evaluationParams['DET_SAMPLE_NAME_2_ID'], True)

    numGlobalCareGt = 0
    numGlobalCareDet = 0

    arrGlobalConfidences = []
    arrGlobalMatches = []

    for resFile in gt:
        gtFile = rrc_evaluation_funcs.decode_utf8(gt[resFile])
        if gtFile is None:
            raise Exception('The file %s is not UTF-8' % resFile)

        recall = 0
        precision = 0
        hmean = 0
        detCorrect = 0
        iouMat = np.empty([1, 1])
        gtPols = []
        detPols = []
        gtTrans = []
        detTrans = []
        gtPolPoints = []
        detPolPoints = []
        gtDontCarePolsNum = []  # Array of Ground Truth Polygons' keys marked as don't Care
        detDontCarePolsNum = []  # Array of Detected Polygons' matched with a don't Care GT
        detMatchedNums = []
        pairs = []

        arrSampleConfidences = []
        arrSampleMatch = []
        sampleAP = 0

        evaluationLog = ''

        pointsList, _, transcriptionsList = rrc_evaluation_funcs.get_tl_line_values_from_file_contents(
            gtFile, evaluationParams['CRLF'], evaluationParams['LTRB'], True, False
        )
        for n in range(len(pointsList)):
            points = pointsList[n]
            transcription = transcriptionsList[n]
            dontCare = transcription == '###'
            if evaluationParams['LTRB']:
                gtRect = Rectangle(*points)
                gtPol = rectangle_to_polygon(gtRect)
            else:
                gtPol = polygon_from_points(points)
            gtPols.append(gtPol)
            gtPolPoints.append(points)

            # On word spotting we will filter some transcriptions with special characters
            if evaluationParams['WORD_SPOTTING']:
                if dontCare == False:
                    if include_in_dictionary(transcription) == False:
                        dontCare = True
                    else:
                        transcription = include_in_dictionary_transcription(transcription)

            gtTrans.append(transcription)
            if dontCare:
                gtDontCarePolsNum.append(len(gtPols) - 1)

        evaluationLog += 'GT polygons: ' + str(
            len(gtPols)
        ) + (' (' + str(len(gtDontCarePolsNum)) + " don't care)\n" if len(gtDontCarePolsNum) > 0 else '\n')

        if resFile in subm:
            detFile = rrc_evaluation_funcs.decode_utf8(subm[resFile])

            pointsList, confidencesList, transcriptionsList = rrc_evaluation_funcs.get_tl_line_values_from_file_contents(
                detFile, evaluationParams['CRLF'], evaluationParams['LTRB'], True, evaluationParams['CONFIDENCES']
            )

            for n in range(len(pointsList)):
                points = pointsList[n]
                transcription = transcriptionsList[n]

                if evaluationParams['LTRB']:
                    detRect = Rectangle(*points)
                    detPol = rectangle_to_polygon(detRect)
                else:
                    detPol = polygon_from_points(points)
                detPols.append(detPol)
                detPolPoints.append(points)
                detTrans.append(transcription)

                if len(gtDontCarePolsNum) > 0:
                    for dontCarePol in gtDontCarePolsNum:
                        dontCarePol = gtPols[dontCarePol]
                        intersected_area = get_intersection(dontCarePol, detPol)
                        pdDimensions = detPol.area()
                        precision = 0 if pdDimensions == 0 else intersected_area / pdDimensions
                        if precision > evaluationParams['AREA_PRECISION_CONSTRAINT']:
                            detDontCarePolsNum.append(len(detPols) - 1)
                            break

            evaluationLog += 'DET polygons: ' + str(
                len(detPols)
            ) + (' (' + str(len(detDontCarePolsNum)) + " don't care)\n" if len(detDontCarePolsNum) > 0 else '\n')

            if len(gtPols) > 0 and len(detPols) > 0:
                # Calculate IoU and precision matrixs
                outputShape = [len(gtPols), len(detPols)]
                iouMat = np.empty(outputShape)
                gtRectMat = np.zeros(len(gtPols), np.int8)
                detRectMat = np.zeros(len(detPols), np.int8)
                for gtNum in range(len(gtPols)):
                    for detNum in range(len(detPols)):
                        pG = gtPols[gtNum]
                        pD = detPols[detNum]
                        iouMat[gtNum, detNum] = get_intersection_over_union(pD, pG)

                for gtNum in range(len(gtPols)):
                    for detNum in range(len(detPols)):
                        if gtRectMat[gtNum] == 0 and detRectMat[
                            detNum] == 0 and gtNum not in gtDontCarePolsNum and detNum not in detDontCarePolsNum:
                            if iouMat[gtNum, detNum] > evaluationParams['IOU_CONSTRAINT']:
                                gtRectMat[gtNum] = 1
                                detRectMat[detNum] = 1
                                # detection matched only if transcription is equal
                                if evaluationParams['WORD_SPOTTING']:
                                    correct = gtTrans[gtNum].upper() == detTrans[detNum].upper()
                                else:
                                    correct = transcription_match(
                                        gtTrans[gtNum].upper(), detTrans[detNum].upper(),
                                        evaluationParams['SPECIAL_CHARACTERS'],
                                        evaluationParams['ONLY_REMOVE_FIRST_LAST_CHARACTER']
                                    ) == True
                                detCorrect += 1 if correct else 0
                                if correct:
                                    detMatchedNums.append(detNum)
                                pairs.append({'gt': gtNum, 'det': detNum, 'correct': correct})
                                evaluationLog += 'Match GT #' + str(gtNum) + ' with Det #' + str(
                                    detNum
                                ) + ' trans. correct: ' + str(correct) + '\n'

            if evaluationParams['CONFIDENCES']:
                for detNum in range(len(detPols)):
                    if detNum not in detDontCarePolsNum:
                        # we exclude the don't care detections
                        match = detNum in detMatchedNums

                        arrSampleConfidences.append(confidencesList[detNum])
                        arrSampleMatch.append(match)

                        arrGlobalConfidences.append(confidencesList[detNum])
                        arrGlobalMatches.append(match)

        numGtCare = len(gtPols) - len(gtDontCarePolsNum)
        numDetCare = len(detPols) - len(detDontCarePolsNum)
        if numGtCare == 0:
            recall = float(1)
            precision = float(0) if numDetCare > 0 else float(1)
            sampleAP = precision
        else:
            recall = float(detCorrect) / numGtCare
            precision = 0 if numDetCare == 0 else float(detCorrect) / numDetCare
            if evaluationParams['CONFIDENCES']:
                sampleAP = compute_ap(arrSampleConfidences, arrSampleMatch, numGtCare)

        hmean = 0 if (precision + recall) == 0 else 2.0 * precision * recall / (precision + recall)

        matchedSum += detCorrect
        numGlobalCareGt += numGtCare
        numGlobalCareDet += numDetCare

        perSampleMetrics[resFile] = {
            'precision': precision,
            'recall': recall,
            'hmean': hmean,
            'pairs': pairs,
            'AP': sampleAP,
            'iouMat': [] if len(detPols) > 100 else iouMat.tolist(),
            'gtPolPoints': gtPolPoints,
            'detPolPoints': detPolPoints,
            'gtTrans': gtTrans,
            'detTrans': detTrans,
            'gtDontCare': gtDontCarePolsNum,
            'detDontCare': detDontCarePolsNum,
            'evaluationParams': evaluationParams,
            'evaluationLog': evaluationLog,
        }

    # Compute AP
    AP = 0
    if evaluationParams['CONFIDENCES']:
        AP = compute_ap(arrGlobalConfidences, arrGlobalMatches, numGlobalCareGt)

    methodRecall = 0 if numGlobalCareGt == 0 else float(matchedSum) / numGlobalCareGt
    methodPrecision = 0 if numGlobalCareDet == 0 else float(matchedSum) / numGlobalCareDet
    methodHmean = 0 if methodRecall + methodPrecision == 0 else 2 * methodRecall * methodPrecision / (
        methodRecall + methodPrecision
    )

    methodMetrics = {'precision': methodPrecision, 'recall': methodRecall, 'hmean': methodHmean, 'AP': AP}

    resDict = {'calculated': True, 'Message': '', 'method': methodMetrics, 'per_sample': perSampleMetrics}

    return resDict


if __name__ == '__main__':
    rrc_evaluation_funcs.main_evaluation(None, default_evaluation_params, validate_data, evaluate_method)



# Copyright (c) Alibaba, Inc. and its affiliates.
from collections import defaultdict
from typing import List, Optional, Union

from evalscope.api.benchmark import BenchmarkMeta, Text2ImageAdapter
from evalscope.api.metric.scorer import AggScore, Score
from evalscope.api.registry import get_metric, register_benchmark
from evalscope.constants import Tags
from evalscope.metrics import mean
from evalscope.utils.function_utils import thread_safe
from evalscope.utils.logger import get_logger

logger = get_logger()


@register_benchmark(
    BenchmarkMeta(
        name='evalmuse',
        pretty_name='EvalMuse',
        dataset_id='AI-ModelScope/T2V-Eval-Prompts',
        description='EvalMuse Text-to-Image Benchmark. Used for evaluating the quality '
        'and semantic alignment of finely generated images',
        tags=[Tags.TEXT_TO_IMAGE],
        subset_list=['EvalMuse'],
        metric_list=['FGA_BLIP2Score'],
        few_shot_num=0,
        train_split=None,
        eval_split='test',
    )
)
class EvalMuseAdapter(Text2ImageAdapter):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        assert len(self.metric_list
                   ) == 1 and self.metric_list[0] == 'FGA_BLIP2Score', 'Only FGA_BLIP2Score is supported for EvalMuse'

    @thread_safe
    def match_score(self, original_prediction, filtered_prediction, reference, task_state):
        # Get prediction and prompt from task state
        image_path = task_state.metadata.get('image_path', original_prediction)

        # Initialize the score object with prediction details
        score = Score(
            extracted_prediction=image_path,
            prediction=image_path,
        )

        # Calculate scores for each configured metric
        try:
            metric_name = self.metric_list[0]
            metric_cls = get_metric(metric_name)
            metric_func = metric_cls()  # Initialize with parameters
            metric_score = metric_func(image_path, task_state.metadata)[0]

            for k, v in metric_score.items():
                score.value[f'{metric_name}:{k}'] = v.cpu().item()
        except Exception as e:
            logger.error(f'Error calculating metric {metric_name}: {e}')
            score.value[metric_name] = 0
            score.metadata[metric_name] = f'error: {str(e)}'

        return score

    def aggregate_scores(self, sample_scores) -> List[AggScore]:
        new_items = defaultdict(list)
        agg_list = []
        for sample_score in sample_scores:
            for metric_name, value in sample_score.score.value.items():
                metrics_prefix = metric_name.split(':')[0]
                category = metric_name.rpartition('(')[-1].split(')')[0]
                category = category.split('-')[0].lower()  # remove the suffix if exists
                new_items[f'{metrics_prefix}:{category}'].append(value)

        for k, v in new_items.items():
            agg_list.append(AggScore(metric_name=k, score=mean(v), num=len(v)))

        return agg_list



# flake8: noqa
FEW_SHOT_SAMPLES = """Answer the following multiple choice question. There is only one correct answer. The last line of your response should be in the format 'ANSWER: $LETTER' (without quotes), where LETTER is one of A, B, C, D, E, F, G, H, I, or J.

Question:
A refracting telescope consists of two converging lenses separated by 100 cm. The eye-piece lens has a focal length of 20 cm. The angular magnification of the telescope is
A) 10
B) 40
C) 6
D) 25
E) 15
F) 50
G) 30
H) 4
I) 5
J) 20

ANSWER: Let's think step by step. In a refracting telescope, if both lenses are converging, the focus of both lenses must be between the two lenses, and thus the focal lengths of the two lenses must add up to their separation. Since the focal length of one lens is 20 cm, the focal length of the other must be 80 cm. The magnification is the ratio of these two focal lengths, or 4.
ANSWER: H.

Question:
Say the pupil of your eye has a diameter of 5 mm and you have a telescope with an aperture of 50 cm. How much more light can the telescope gather than your eye?
A) 1000 times more
B) 50 times more
C) 5000 times more
D) 500 times more
E) 10000 times more
F) 20000 times more
G) 2000 times more
H) 100 times more
I) 10 times more
J) N/A

ANSWER: Let's think step by step. The amount of light a telescope can gather compared to the human eye is proportional to the area of its apertures. The area of a circle is given by the formula $A = \pi \left(\frac{{D}}{{2}}\right)^2$, where $D$ is the diameter. Therefore, the relative light-gathering power is calculated as:
\[
\frac{{\left(\frac{{50 \text{{ cm}}}}{{2}}\right)^2}}{{\left(\frac{{5 \text{{ mm}}}}{{2}}\right)^2}} = \frac{{\left(\frac{{50 \text{{ cm}}}}{{0.1 \text{{ cm}}}}\right)^2}}{{\left(\frac{{5 \text{{ mm}}}}{{0.1 \text{{ cm}}}}\right)^2}} = \frac{{500^2}}{{5^2}} = 10000.
\]
ANSWER: E.

Question:
Where do most short-period comets come from and how do we know?
A) The Kuiper belt; short period comets tend to be in the plane of the solar system like the Kuiper belt.
B) The asteroid belt; short period comets tend to come from random directions indicating a spherical distribution of comets called the asteroid belt.
C) The asteroid belt; short period comets tend to be in the plane of the solar system just like the asteroid belt.
D) The Oort cloud; short period comets have orbital periods similar to asteroids like Vesta and are found in the plane of the solar system just like the Oort cloud.
E) The Oort Cloud; short period comets tend to come from random directions indicating a spherical distribution of comets called the Oort Cloud.
F) The Oort cloud; short period comets tend to be in the plane of the solar system just like the Oort cloud.
G) The asteroid belt; short period comets have orbital periods similar to asteroids like Vesta and are found in the plane of the solar system just like the asteroid belt.

ANSWER: Let's think step by step. Most short-period comets originate from the Kuiper belt. This is deduced from the observation that these comets tend to follow orbits that lie in the plane of the solar system, similar to the distribution of objects in the Kuiper belt itself. Thus, the alignment of these cometary orbits with the ecliptic plane points to their Kuiper belt origin.
ANSWER: A.

Question:
Colors in a soap bubble result from light
A) dispersion
B) deflection
C) refraction
D) reflection
E) interference
F) converted to a different frequency
G) polarization
H) absorption
I) diffraction
J) transmission

ANSWER: Let's think step by step. The colorful patterns observed in a soap bubble are caused by the phenomenon of light interference. This occurs when light waves bounce between the two surfaces of the soap film, combining constructively or destructively based on their phase differences and the varying thickness of the film. These interactions result in vibrant color patterns due to variations in the intensity of different wavelengths of light.
ANSWER: E.

Question:
A microwave oven is connected to an outlet, 120 V, and draws a current of 2 amps. At what rate is energy being used by the microwave oven?
A) 240 W
B) 120 W
C) 10 W
D) 480 W
E) 360 W
F) 200 W
G) 30 W
H) 150 W
I) 60 W
J) 300 W

ANSWER: Let's think step by step. The rate of energy usage, known as power, in an electrical circuit is calculated by the product of voltage and current. For a microwave oven connected to a 120 V outlet and drawing a current of 2 amps, the power consumption can be calculated as follows:
\[
\text{{Power}} = \text{{Voltage}} \times \text{{Current}} = 120 \, \text{{V}} \times 2 \, \text{{A}} = 240 \, \text{{W}}.
\]
Therefore, the microwave oven uses energy at a rate of 240 watts.
ANSWER: A.

"""



from .generate_config import GenerateConfig
from .model import Model, ModelAPI, get_model, get_model_with_task_config
from .model_output import (
    ChatCompletionChoice,
    Logprob,
    Logprobs,
    ModelOutput,
    ModelUsage,
    StopReason,
    TopLogprob,
    as_stop_reason,
)



# Copyright (c) Alibaba, Inc. and its affiliates.

from typing import Any, Dict, List, Tuple

from evalscope.api.benchmark import BenchmarkMeta, DefaultDataAdapter
from evalscope.api.dataset import Sample
from evalscope.api.dataset.dataset import DatasetDict, MemoryDataset
from evalscope.api.metric.scorer import AggScore, SampleScore, Score
from evalscope.api.registry import register_benchmark
from evalscope.constants import Tags
from evalscope.report.report import Report, Subset
from evalscope.utils.logger import get_logger

logger = get_logger()

SUBSET_LIST = [
    'en', 'zh', 'ar', 'bn', 'de', 'es', 'fr', 'id', 'it', 'ja', 'ko', 'ms', 'pt', 'ru', 'sw', 'te', 'th', 'vi'
]
LEVEL_LIST = ['low', 'medium', 'high', 'top']


@register_benchmark(
    BenchmarkMeta(
        name='poly_math',
        pretty_name='PolyMath',
        tags=[Tags.MATH, Tags.REASONING, Tags.MULTI_LINGUAL],
        description=
        'PolyMath is a multilingual mathematical reasoning benchmark covering 18 languages and 4 easy-to-hard difficulty levels, with 9,000 high-quality problem samples. Our benchmark ensures difficulty comprehensiveness, language diversity, and high-quality translation, making it a highly discriminative multilingual mathematical benchmark in the era of reasoning LLMs.',  # noqa: E501
        dataset_id='evalscope/PolyMath',
        subset_list=SUBSET_LIST,
        metric_list=[{
            'acc': {
                'numeric': True
            }
        }],
        eval_split='test',
        prompt_template='{question}',
    )
)
class PolyMathAdapter(DefaultDataAdapter):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def load(self) -> Tuple[DatasetDict, None]:
        """Load all difficulty levels, rename subsets with their level suffix, and merge them."""
        # Need to load all levels to get the full dataset
        dataset_list: List[Dict[str, MemoryDataset]] = []
        original_split = getattr(self, 'eval_split', None)
        try:
            for level in LEVEL_LIST:
                self.eval_split = level
                cur_level_dataset_dict, _ = super().load()
                # Build a renamed mapping without mutating during iteration
                renamed: Dict[str, MemoryDataset] = {
                    f'{subset}-{level}': ds
                    for subset, ds in cur_level_dataset_dict.items()
                }
                dataset_list.append(renamed)
        finally:
            # Restore original split to avoid side effects
            if original_split is not None:
                self.eval_split = original_split
        # Merge all levels into one dataset
        return DatasetDict.from_dataset_dicts(dataset_list), None

    def record_to_sample(self, record: Dict[str, Any]) -> Sample:
        """Convert a single record into a Sample with language-specific instruction."""
        from .utils.instruction import query_dic

        # e.g., 'high-en-1'
        question_id = record['id']
        level, language, index = question_id.split('-')
        # Get the instruction for the specific language
        instruction = query_dic[language]

        return Sample(
            input=record['question'] + '\n' + instruction,
            target=record['answer'],
            metadata={
                'level': level,
                'language': language,
                'index': index,
            },
        )

    def _on_generate_report_end(self, report: Report, output_dir, **kwargs):
        """
        Finalize the report generation process. Calculate the difficulty-weighted accuracy (DW-ACC)
        per language and the overall DW-ACC, and append as a new category to each metric.
        """
        from evalscope.report import Category, Metric

        WEIGHT_DENOMINATOR = 15  # 1 + 2 + 4 + 8 for ['low','medium','high','top']

        for metric in report.metrics:
            # Collect all subsets by name for easy lookup (e.g., "en-low")
            subset_dict: Dict[str, Subset] = {}
            for category in metric.categories:
                for subset in category.subsets:
                    subset_dict[subset.name] = subset

            # Compute per-language DW-ACC
            dw_subsets: List[Subset] = []
            for language in SUBSET_LIST:
                weighted_sum = 0.0
                total_num = 0
                for i, level in enumerate(LEVEL_LIST):
                    s = subset_dict.get(f'{language}-{level}')
                    if s is not None:
                        weighted_sum += (2**i) * s.score
                        total_num += s.num
                # missing subsets contribute 0 by design
                if total_num == 0:
                    continue
                dw_acc = weighted_sum / WEIGHT_DENOMINATOR
                dw_subsets.append(Subset(name=language, score=dw_acc, num=total_num))

            # Overall DW-ACC: unweighted average across languages
            if dw_subsets:
                overall_score = sum(s.score for s in dw_subsets) / len(dw_subsets)
                overall_num = sum(s.num for s in dw_subsets)
                dw_subsets.append(Subset(name='OVERALL', score=overall_score, num=overall_num))

        # Append DW-ACC metric
        if dw_subsets:
            report.metrics.append(Metric(name='DW-ACC', categories=[Category(name='-', subsets=dw_subsets)]))



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright (c) EleutherAI Inc, and its affiliates.

from typing import Any, Dict

from evalscope.api.benchmark import BenchmarkMeta, DefaultDataAdapter
from evalscope.api.dataset import Sample
from evalscope.api.evaluator import TaskState
from evalscope.api.metric import Score
from evalscope.api.registry import register_benchmark
from evalscope.constants import Tags
from evalscope.utils.logger import get_logger

logger = get_logger()

PROMPT_TEMPLATE = """
Read the content and answer the following question.

Content: {content}

Question: {question}

Keep your The last line of your response should be of the form "ANSWER: $ANSWER" (without quotes) where $ANSWER is the answer to the problem.
""".lstrip()  # noqa: E501


@register_benchmark(
    BenchmarkMeta(
        name='trivia_qa',
        pretty_name='TriviaQA',
        dataset_id='evalscope/trivia_qa',
        tags=[Tags.QA, Tags.READING_COMPREHENSION],
        description=
        'TriviaQA is a large-scale reading comprehension dataset consisting of question-answer pairs collected from trivia websites. It includes questions with multiple possible answers, making it suitable for evaluating the ability of models to understand and generate answers based on context.',  # noqa: E501
        subset_list=['rc.wikipedia'],
        few_shot_num=0,
        train_split=None,
        eval_split='validation',
        metric_list=[{
            'acc': {
                'allow_inclusion': True
            }
        }],
        prompt_template=PROMPT_TEMPLATE,
    )
)
class TriviaQaAdapter(DefaultDataAdapter):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def record_to_sample(self, record: Dict[str, Any]) -> Sample:
        question = record['question']
        answers = record['answer']['aliases'] + record['answer']['normalized_aliases']
        content = record['entity_pages']['wiki_context']
        return Sample(
            input=question, target=answers, metadata={
                'question_id': record['question_id'],
                'content': content
            }
        )

    def format_prompt_template(self, sample):
        return self.prompt_template.format(content=sample.metadata['content'], question=sample.input)

    def extract_answer(self, prediction: str, task_state: TaskState):
        # use regex to extract the answer from the prediction
        import re

        pattern = r'ANSWER:\s*(.*)'
        match = re.search(pattern, prediction)
        if match:
            return match.group(1).strip()
        return prediction.strip()



import jieba
import nltk
import re
from nltk.metrics import f_measure, precision, recall
from nltk.translate import meteor_score

from evalscope.utils.function_utils import thread_safe


def contain_chinese_string(text):
    chinese_pattern = re.compile(r'[\u4e00-\u9fa5]')
    return bool(chinese_pattern.search(text))


@thread_safe
def cal_per_metrics(pred, gt):
    metrics = {}

    if contain_chinese_string(gt) or contain_chinese_string(pred):
        reference = jieba.lcut(gt)
        hypothesis = jieba.lcut(pred)
    else:
        reference = gt.split()
        hypothesis = pred.split()

    metrics['bleu'] = nltk.translate.bleu([reference], hypothesis)
    metrics['meteor'] = meteor_score.meteor_score([reference], hypothesis)

    reference = set(reference)
    hypothesis = set(hypothesis)
    metrics['f_measure'] = f_measure(reference, hypothesis)

    metrics['precision'] = precision(reference, hypothesis)
    metrics['recall'] = recall(reference, hypothesis)
    metrics['edit_dist'] = nltk.edit_distance(pred, gt) / max(len(pred), len(gt))
    return metrics


if __name__ == '__main__':
    # Examples for region text recognition and read all text tasks
    predict_text = "metrics['edit_dist'] = nltk.edit_distance(pred, gt) / max(len(pred), len(gt))"
    true_text = 'metrics = nltk.edit_distance(pred, gt) / max(len(pred), len(gt))'

    scores = cal_per_metrics(predict_text, true_text)

    predict_text = "metrics['edit_dist'] len(gt))"
    true_text = 'metrics = nltk.edit_distance(pred, gt) / max(len(pred), len(gt))'

    scores = cal_per_metrics(predict_text, true_text)
    print(scores)



import abc
import random
from collections import defaultdict
from dataclasses import dataclass, field
from pydantic import BaseModel, Field
from typing import Any, Callable, Dict, Iterator, List, Optional, Sequence, Union

from evalscope.api.messages import ChatMessage, messages_to_markdown
from evalscope.api.tool import ToolInfo


class Sample(BaseModel):
    r"""Sample for an evaluation task."""

    input: Union[str, List[ChatMessage]]
    """The input to be submitted to the model."""

    choices: Optional[List[str]] = None
    """List of available answer choices (used only for multiple-choice evals)."""

    target: Union[str, List[str]] = ''
    """Ideal target output. May be a literal value or narrative text to be used by a model grader."""

    id: Optional[int] = None
    """Unique identifier for sample."""

    group_id: Optional[int] = None
    """Identifier for the group this sample belongs to, used for grouping k repeated samples."""

    tools: Optional[List[ToolInfo]] = None
    """List of tools available to the model during inference (optional)."""

    subset_key: Optional[str] = None
    """Key for the subset this sample belongs to, used for generating subsets (optional)."""

    metadata: Dict[str, Any] = Field(default_factory=dict)
    """Arbitrary metadata associated with the sample."""

    sandbox: Optional[str] = None
    """Sandbox environment type and optional config file."""

    files: Optional[Dict[str, str]] = None
    """Files that go along with the sample (copied to SandboxEnvironment)"""

    setup: Optional[str] = None
    """Setup script to run for sample (run within default SandboxEnvironment)."""

    def pretty_print(self) -> str:
        """Return a pretty-printed string representation of the sample."""
        if isinstance(self.input, str):
            input_text = self.input
        else:
            input_text = messages_to_markdown(self.input, max_length=50)
        return f'Sample ID: {self.id}\nInput: {input_text}\nTarget: {self.target}'


@dataclass
class FieldSpec:
    r"""Specification for mapping data source fields to sample fields."""

    input: str = field(default='input')
    """Name of the field containing the sample input."""

    target: str = field(default='target')
    """Name of the field containing the sample target."""

    choices: str = field(default='choices')
    """Name of field containing the list of answer choices."""

    id: int = field(default=0)
    """ Unique identifier for the sample."""

    metadata: Optional[List[str]] = field(default=None)
    """List of additional field names that should be read as metadata."""

    sandbox: str = field(default='sandbox')
    """Sandbox type along with optional config file."""

    files: str = field(default='files')
    """Files that go along with the sample."""

    setup: str = field(default='setup')
    """Setup script to run for sample (run within default SandboxEnvironment)."""


class Dataset(Sequence[Sample], abc.ABC):
    r"""A sequence of Sample objects.

    Datasets provide sequential access (via conventional indexes or slicing)
    to a collection of Sample objects.
    """

    @property
    @abc.abstractmethod
    def name(self) -> Optional[str]:
        ...

    @property
    @abc.abstractmethod
    def location(self) -> Optional[str]:
        ...

    @property
    @abc.abstractmethod
    def shuffled(self) -> bool:
        ...

    @abc.abstractmethod
    def __iter__(self) -> Iterator[Sample]:
        """Return an iterator over the samples."""
        ...

    @abc.abstractmethod
    def __getitem__(self, index: Union[int, slice]) -> Union[Sample, 'Dataset']:
        ...

    @abc.abstractmethod
    def __len__(self) -> int:
        ...

    @abc.abstractmethod
    def filter(self, predicate: Callable[[Sample], bool], name: Optional[str] = None) -> 'Dataset':
        """Filter the dataset using a predicate. Only samples matching the predicate will be included.

        Args:
          predicate: Filtering function.
          name: Name for filtered dataset (optional).

        Returns:
          Filtered dataset.
        """
        ...

    @abc.abstractmethod
    def shuffle(self, seed: Optional[int] = None) -> None:
        """Shuffle the order of the dataset (in place).

        Args:
           seed: Random seed for shuffling (optional).
        """
        ...

    @abc.abstractmethod
    def shuffle_choices(self, seed: Optional[int] = None) -> None:
        """Shuffle the order of the choices with each sample.

        Args:
           seed: Random seed for shuffling (optional).
        """
        ...

    @abc.abstractmethod
    def reindex(self, group_size=1):
        """Reindex the dataset samples to ensure consistent ordering.

        Args:
           group_size: Number of samples per group for setting group_id.
        """
        ...


class MemoryDataset(Dataset):
    r"""A Dataset stored in memory."""

    def __init__(
        self,
        samples: List[Sample],
        name: Optional[str] = None,
        location: Optional[str] = None,
        shuffled: bool = False,
    ) -> None:
        r"""A dataset of samples held in an in-memory list.

        Datasets provide sequential access (via conventional indexes or slicing)
        to a collection of Sample objects. The ListDataset is explicitly
        initialized with a list that is held in memory.

        Args:
            samples (List[Sample]): The list of sample objects.
            name (str | None): Optional name for dataset.
            location (str | None): Optional location for dataset.
            shuffled (bool): Was the dataset shuffled after reading.
        """
        self.samples = samples
        self._name = name
        self._location = location
        self._shuffled = shuffled

    @property
    def name(self) -> Optional[str]:
        """Dataset name."""
        return self._name

    @property
    def location(self) -> Optional[str]:
        """Dataset location."""
        return self._location

    @property
    def shuffled(self) -> bool:
        """Was the dataset shuffled."""
        return self._shuffled

    def __iter__(self) -> Iterator[Sample]:
        return iter(self.samples)

    def __getitem__(self, index: Union[int, slice]) -> Union[Sample, Dataset]:
        if isinstance(index, int):
            return self.samples[index]
        else:
            return MemoryDataset(
                samples=self.samples[index],
                name=self.name,
                location=self.location,
                shuffled=self.shuffled,
            )

    def __len__(self) -> int:
        return len(self.samples)

    def shuffle(self, seed: Optional[int] = None) -> None:
        if seed is not None:
            random.Random(seed).shuffle(self.samples)
        else:
            random.shuffle(self.samples)
        self._shuffled = True

    def shuffle_choices(self, seed: Optional[int] = None) -> None:
        from evalscope.utils.multi_choices import answer_character

        rand = random.Random(seed)
        for sample in self.samples:
            if not sample.choices:
                continue
            # The original positions
            positions = list(range(len(sample.choices)))

            # Shuffle the choices
            rand.shuffle(positions)
            shuffled_choices = [sample.choices[i] for i in positions]

            # Map of original position / target letter
            position_map = {i: answer_character(new_i) for new_i, i in enumerate(positions)}

            # Update to the shuffled choices and target
            sample.choices = shuffled_choices
            sample.target = self._remap_target(sample.target, position_map=position_map)

    def _remap_target(self, target: Union[str, List[str]], position_map: Dict[int, str]) -> Union[str, List[str]]:
        from evalscope.utils.multi_choices import answer_index

        if isinstance(target, list):
            return [position_map[answer_index(t)] for t in target]
        else:
            return position_map[answer_index(target)]

    def filter(self, predicate: Callable[[Sample], bool], name: Optional[str] = None) -> 'MemoryDataset':
        return MemoryDataset(
            name=name or self.name,
            location=self.location,
            samples=[sample for sample in self.samples if predicate(sample)],
            shuffled=self.shuffled,
        )

    def reindex(self, group_size=1):
        # Reindex the dataset samples to ensure consistent ordering
        for i, sample in enumerate(self.samples):
            sample.id = i
            sample.group_id = i // group_size


class DatasetDict:
    """
    A dictionary-like container for datasets.
    """

    def __init__(self, datasets: Dict[str, Dataset]):
        self.datasets = datasets

    def __getitem__(self, key: str) -> Dataset:
        return self.datasets[key]

    def __setitem__(self, key: str, value: Dataset) -> None:
        self.datasets[key] = value

    def __delitem__(self, key: str) -> None:
        del self.datasets[key]

    def get(self, key: str, default: Optional[Dataset] = None) -> Optional[Dataset]:
        return self.datasets.get(key, default)

    def items(self):
        return self.datasets.items()

    def keys(self):
        return self.datasets.keys()

    def values(self):
        return self.datasets.values()

    def __len__(self) -> int:
        return len(self.datasets)

    @classmethod
    def from_dataset(
        cls,
        dataset: Dataset,
        subset_list: List[str],
        limit: Optional[Union[int, float]] = None,
        repeats: int = 1
    ) -> 'DatasetDict':
        """
        Create a DatasetDict from a single Dataset using subset key in the sample.

        Args:
            dataset (Dataset): The dataset to wrap in a DatasetDict.
            subset_list (List[str]): List of subset keys to include.
            limit (int | float | None): Optional limit on number of samples per subset.
                If int, limits to that many samples. If float, limits to that fraction of samples.

        Returns:
            DatasetDict: A new DatasetDict containing the provided dataset.
        """
        data_dict = defaultdict(list)
        dataset_dict = defaultdict(list)
        # init subset keys to prevent order issues
        for key in subset_list:
            data_dict[key] = []
            dataset_dict[key] = []

        # Loop through each sample in the dataset
        for sample in dataset.samples:
            subset_key = sample.subset_key or 'default'
            data_dict[subset_key].append(sample)
        # Create a MemoryDataset for each subset key
        for key, samples in data_dict.items():
            if key not in subset_list:
                continue
            # Apply limit if specified
            if limit is not None:
                if isinstance(limit, float):
                    limit = int(len(samples) * limit)
                total_limit = limit * repeats
                samples = samples[:total_limit]
            cur_dataset = MemoryDataset(samples, name=dataset.name)
            # Reindex the dataset to ensure consistent IDs and group IDs
            cur_dataset.reindex(group_size=repeats)
            dataset_dict[key] = cur_dataset
        return cls(dataset_dict)

    @classmethod
    def from_dataset_dicts(cls, dataset_dicts: List['DatasetDict']) -> 'DatasetDict':
        """
        Create a DatasetDict by merging multiple DatasetDicts.

        Args:
            dataset_dicts (List[DatasetDict]): List of DatasetDicts to merge.

        Returns:
            DatasetDict: A new DatasetDict containing the merged datasets.
        """
        merged_dict = defaultdict(list)
        for dataset_dict in dataset_dicts:
            for key, dataset in dataset_dict.items():
                merged_dict[key].extend(dataset.samples)
        # Create a MemoryDataset for each subset key
        final_dict = {}
        for key, samples in merged_dict.items():
            final_dict[key] = MemoryDataset(samples, name=key)
        return cls(final_dict)



import ast
import os
import re
import shutil
import zipfile

from evalscope.constants import DEFAULT_EVALSCOPE_CACHE_DIR
from evalscope.utils.function_utils import thread_safe
from .spotting_eval import rrc_evaluation_funcs_1_1 as rrc_evaluation_funcs
from .spotting_eval.script import default_evaluation_params, evaluate_method, validate_data


def extract_bounding_boxes_robust(predict_str):
    """
    Extract coordinates and text content from the given prediction string,
    handling potential format issues.

    Args:
    predict_str (str): Model prediction output as a string.

    Returns:
    list: Extracted data in the format [[x1, y1, x2, y2, text_content], ...].
          Returns None if no valid data is extracted.
    """
    results = []
    seen = set()

    # try parsing with ast.literal_eval
    try:
        data = ast.literal_eval(predict_str)
    except Exception:
        data = None

    if data is not None:
        if isinstance(data, (list, tuple)):
            for item in data:
                if isinstance(item, (list, tuple)) and len(item) >= 5:
                    x1_str, y1_str, x2_str, y2_str = item[:4]
                    text_content = item[4]

                    x1_str = str(x1_str).strip()
                    y1_str = str(y1_str).strip()
                    x2_str = str(x2_str).strip()
                    y2_str = str(y2_str).strip()
                    text_content = str(text_content).replace('\n', '').strip().strip('"').strip("'")

                    try:
                        x1 = int(x1_str)
                        y1 = int(y1_str)
                        x2 = int(x2_str)
                        y2 = int(y2_str)

                        if not (0 <= x1 <= 1000 and 0 <= y1 <= 1000 and 0 <= x2 <= 1000 and 0 <= y2 <= 1000):
                            continue

                        key = (x1, y1, x2, y2, text_content)
                        if key in seen:
                            continue

                        seen.add(key)
                        results.append([x1, y1, x2, y2, text_content])
                    except ValueError:
                        continue
    else:
        # try parsing with regular expression

        list_content = predict_str
        items = re.findall(r'[\[\(]\s*([^\[\]\(\)]*?)\s*[\]\)]', list_content)

        if not items:
            return None

        for item in items:
            parts = item.split(',', 4)
            if len(parts) < 5:
                continue

            x1_str, y1_str, x2_str, y2_str, text_content = parts

            x1_str = x1_str.strip()
            y1_str = y1_str.strip()
            x2_str = x2_str.strip()
            y2_str = y2_str.strip()
            text_content = text_content.replace('\n', '').strip().strip('"').strip("'")

            try:
                x1 = int(x1_str)
                y1 = int(y1_str)
                x2 = int(x2_str)
                y2 = int(y2_str)

                if not (0 <= x1 <= 1000 and 0 <= y1 <= 1000 and 0 <= x2 <= 1000 and 0 <= y2 <= 1000):
                    continue

                key = (x1, y1, x2, y2, text_content)
                if key in seen:
                    continue

                seen.add(key)
                results.append([x1, y1, x2, y2, text_content])
            except ValueError:
                continue

    if not results:
        return None

    return results


def zip_folder(source_folder, destination_zip):
    abs_source = os.path.abspath(source_folder)
    abs_destination = os.path.abspath(destination_zip)

    with zipfile.ZipFile(abs_destination, 'w', zipfile.ZIP_DEFLATED) as zf:
        for root, _, files in os.walk(abs_source):
            for file in files:
                abs_file_path = os.path.join(root, file)

                relative_path = os.path.relpath(abs_file_path, abs_source)
                zf.write(abs_file_path, relative_path)


@thread_safe
def spotting_evaluation(prediction_list, img_metas):
    score = 0

    submit_path = os.path.join(DEFAULT_EVALSCOPE_CACHE_DIR, 'eval', 'ocrbench_v2', 'submit')
    gt_path = os.path.join(DEFAULT_EVALSCOPE_CACHE_DIR, 'eval', 'ocrbench_v2', 'gt')
    submit_zip_path = os.path.join(DEFAULT_EVALSCOPE_CACHE_DIR, 'eval', 'ocrbench_v2', 'submit.zip')
    gt_zip_path = os.path.join(DEFAULT_EVALSCOPE_CACHE_DIR, 'eval', 'ocrbench_v2', 'gt.zip')
    for file_path in [submit_path, gt_path, submit_zip_path, gt_zip_path]:
        if 'zip' in file_path:
            if os.path.exists(file_path):
                os.remove(file_path)
        else:
            if os.path.exists(file_path):
                shutil.rmtree(file_path)
            os.makedirs(file_path, exist_ok=True)

    res_submit_list = []
    for item in prediction_list:
        x1, y1, x2, y2, rec = item
        if x1 >= x2 or y1 >= y2:
            continue

        res_submit_list.append(','.join([str(x1), str(y1), str(x2), str(y1), str(x2), str(y2), str(x1), str(y2), rec]))

    res_gt_list = []
    for bbox, rec in zip(img_metas['bbox_list'], img_metas['content']):
        x_coords = bbox[0::2]
        y_coords = bbox[1::2]

        x1, y1 = min(x_coords), min(y_coords)
        x2, y2 = max(x_coords), max(y_coords)

        res_gt_list.append(','.join([str(x1), str(y1), str(x2), str(y1), str(x2), str(y2), str(x1), str(y2), rec]))

    if len(res_submit_list) == 0 or len(res_gt_list) == 0:
        return 0

    with open(os.path.join(submit_path, 'res_img_0.txt'), 'w') as f:
        for item in res_submit_list[:-1]:
            f.write(item + '\n')
        f.write(res_submit_list[-1])

    with open(os.path.join(gt_path, 'gt_img_0.txt'), 'w') as f:
        for item in res_gt_list[:-1]:
            f.write(item + '\n')
        f.write(res_gt_list[-1])

    zip_folder(submit_path, submit_zip_path)
    zip_folder(gt_path, gt_zip_path)

    command = {'g': gt_zip_path, 's': submit_zip_path, 'o': DEFAULT_EVALSCOPE_CACHE_DIR, 'p': '{"IOU_CONSTRAINT":0.5}'}

    # run rrc_evaluation_funcs
    result = rrc_evaluation_funcs.main_evaluation(command, default_evaluation_params, validate_data, evaluate_method)
    score = result['method']['hmean']
    return score



import json
import os
from typing import Any, Dict, Iterator, List

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.datasets.base import DatasetPluginBase
from evalscope.perf.plugin.registry import register_dataset


@register_dataset('openqa')
class OpenqaDatasetPlugin(DatasetPluginBase):
    """Read dataset and return prompt.
    Datasets: https://www.modelscope.cn/datasets/AI-ModelScope/HC3-Chinese/resolve/master/open_qa.jsonl
    """

    def __init__(self, query_parameters: Arguments):
        super().__init__(query_parameters)

    def build_messages(self) -> Iterator[List[Dict]]:
        if not self.query_parameters.dataset_path:
            from modelscope import dataset_snapshot_download

            file_name = 'open_qa.jsonl'
            local_path = dataset_snapshot_download('AI-ModelScope/HC3-Chinese', allow_patterns=[file_name])
            self.query_parameters.dataset_path = os.path.join(local_path, file_name)

        for item in self.dataset_line_by_line(self.query_parameters.dataset_path):
            item = json.loads(item)
            prompt = item['question'].strip()
            is_valid, _ = self.check_prompt_length(prompt)
            if is_valid:
                if self.query_parameters.apply_chat_template:
                    message = self.create_message(prompt)
                    yield [message]
                else:
                    yield prompt



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Any, Dict, List, Optional, Tuple

from evalscope.utils import get_logger
from . import ifeval

logger = get_logger()


def gen_acc_strict(x: Dict[str, Any]) -> Dict[str, List]:
    # reference: fbcode/gen_ai/github/fair_evals/evals/tasks/finetune/ifeval.py
    response = str(x['response'])
    instruction_list = x['instruction_id_list']
    is_following_list = []
    for index, instruction_id in enumerate(instruction_list):
        instruction_cls = ifeval.INSTRUCTION_DICT[instruction_id]
        instruction = instruction_cls(instruction_id)

        instruction.build_description(**x['kwargs'][index])
        if response and instruction.check_following(response):
            is_following_list.append(True)
        else:
            is_following_list.append(False)

    return {
        'follow_instruction_list': is_following_list,
        'instruction_id_list': instruction_list,
    }


def gen_acc_loose(x: Dict[str, Any]) -> Dict[str, List]:
    response = str(x['response'])
    r = response.split('\n')
    response_remove_first = '\n'.join(r[1:]).strip()
    response_remove_last = '\n'.join(r[:-1]).strip()
    response_remove_both = '\n'.join(r[1:-1]).strip()
    revised_response = response.replace('*', '')
    revised_response_remove_first = response_remove_first.replace('*', '')
    revised_response_remove_last = response_remove_last.replace('*', '')
    revised_response_remove_both = response_remove_both.replace('*', '')
    all_responses = [
        response,
        revised_response,
        response_remove_first,
        response_remove_last,
        response_remove_both,
        revised_response_remove_first,
        revised_response_remove_last,
        revised_response_remove_both,
    ]
    instruction_list = x['instruction_id_list']
    is_following_list = []
    for index, instruction_id in enumerate(instruction_list):
        instruction_cls = ifeval.INSTRUCTION_DICT[instruction_id]
        instruction = instruction_cls(instruction_id)

        instruction.build_description(**x['kwargs'][index])

        is_following = False
        for r in all_responses:  # type: ignore
            if r.strip() and instruction.check_following(r):  # type: ignore
                is_following = True
                break

        is_following_list.append(is_following)
    return {
        'follow_instruction_list': is_following_list,
        'instruction_id_list': instruction_list,
    }


def parse_result(outputs: List[Dict[str, Any]]) -> Tuple[float, float]:

    prompt_total = 0
    prompt_correct = 0
    instruction_total = 0
    instruction_correct = 0

    for example in outputs:
        follow_instruction_list = example['follow_instruction_list']
        instruction_id_list = example['instruction_id_list']

        prompt_total += 1
        if all(follow_instruction_list):
            prompt_correct += 1

        instruction_total += len(instruction_id_list)
        instruction_correct += sum(follow_instruction_list)

    return prompt_correct / prompt_total if prompt_total > 0 else 0, \
        instruction_correct / instruction_total if instruction_total > 0 else 0


def parse_result_no_reduce(outputs: List[Dict[str, Any]]) -> Tuple[List, List]:

    prompt_res = []
    inst_res = []

    for example in outputs:
        follow_instruction_list = example['follow_instruction_list']
        instruction_id_list = example['instruction_id_list']
        if all(follow_instruction_list):
            prompt_res.append(1)
        else:
            prompt_res.append(0)
        inst_res.append(sum(follow_instruction_list) / len(instruction_id_list) if instruction_id_list else 0.0)

    return prompt_res, inst_res



import re
from typing import Any, Dict

from evalscope.api.benchmark import BenchmarkMeta, DefaultDataAdapter
from evalscope.api.dataset import Sample
from evalscope.api.evaluator import TaskState
from evalscope.api.metric import Score
from evalscope.api.registry import register_benchmark
from evalscope.constants import Tags
from evalscope.utils.logger import get_logger

logger = get_logger()

GRADER_TEMPLATE = """
请根据给定问题、标准答案和模型预测的答案来评估模型的回答是否正确。您的任务是将结果评定为：【正确】、【错误】或【未尝试】。

首先，我们将列出每个评定类别的示例，然后请您对新问题的预测答案进行评定。
以下是【正确】的答复示例：
```
问题：贝拉克·奥巴马的孩子叫什么名字？
标准答案：玛丽亚·奥巴马和萨莎·奥巴马
模型预测1：Malia Obama and Sasha Obama
模型预测2：玛丽亚和萨沙
模型预测3：大多数人会说是玛丽亚和萨莎，但我不确定，需要再确认
模型预测4：巴拉克·奥巴马有两个女儿，她们分别是玛丽亚·安和娜塔莎·玛丽安，但通常称作玛丽亚·奥巴马和萨莎·奥巴马。玛丽亚出生于1998年7月4日，萨莎出生于2001年6月10日。
```
这些答复均为【正确】，因为：
    - 完整地包含了标准答案中的重要信息。
    - 不包含任何与标准答案矛盾的信息。
    - 只关注语义内容，中英文，大小写、标点、语法和顺序不重要。
    - 答复中出现模糊语句或猜测是可以接受的，前提是包含了标准答案且不含有不正确信息或矛盾。

以下是【错误】的答复示例：
```
问题：巴拉克·奥巴马的孩子叫什么名字？
标准答案：玛丽亚·奥巴马和萨莎·奥巴马
模型预测1：玛丽亚
模型预测2：玛丽亚、萨莎和苏珊
模型预测3：巴拉克·奥巴马没有孩子
模型预测4：我认为是玛丽亚和萨莎。或者是玛丽亚和杰基。或者是乔伊和玛丽亚。
模型预测5：虽然我不知道他们的确切名字，但能说出巴拉克·奥巴马有三个孩子。
模型预测6：你可能是想说贝茜和奥利维亚。不过您应通过最新的参考资料确认详细信息。那是正确的答案吗？
```
这些答复均为【错误】，因为：
    - 答复中包含与标准答案矛盾的事实陈述。即使在陈述中略带保留（例如："可能是"，"虽然我不确定，但我认为"），也视为错误。

以下是【未尝试】的答复示例：
```
问题：巴拉克·奥巴马的孩子叫什么名字？
标准答案：玛丽亚·奥巴马和萨莎·奥巴马
模型预测1：我不知道。
模型预测2：我需要更多关于您所指奥巴马的上下文。
模型预测3：不查阅网络我无法回答这个问题，不过我知道巴拉克·奥巴马有两个孩子。
模型预测4：巴拉克·奥巴马有两个孩子。我知道其中一个叫玛丽亚，但我不确定另一个的名字。
```
这些答复均为【未尝试】，因为：
    - 没有包含标准答案中的重要信息。
    - 回复中没有与标准答案矛盾的陈述。

另外注意以下几点：
- 对于标准答案为数字的问题，预测答案应和标准答案一致。例如，考虑问题"金山铁路黄浦江特大桥的全长是多少米？"，标准答案为"3518.17"：
    - 预测答案"3518"、"3518.1"、"3518.17"均为【正确】。
    - 预测答案"3520"和"3600"均为【错误】。
    - 预测答案"大约3500米"和"超过3000米"被视为【未尝试】，因为它们既不确认也不与标准答案矛盾。
- 如果标准答案包含比问题更多的信息，预测答案只需包含问题中提到的信息。
    - 例如，考虑问题"菱镁矿的主要化学成分是什么？"标准答案为"碳酸镁（MgCO3）"。"碳酸镁"或"MgCO3"均视为【正确】答案。
- 如果从问题中明显可以推断出预测答案省略的信息，那么算作正确。
    - 例如，问题"巴鲁米尼的努拉吉遗迹在1997年被联合国教科文组织列为世界文化遗产，那么这遗址在哪个地区？"标准答案为"意大利撒丁岛"，预测答案"撒丁岛"被视为【正确】。
- 如果能明显看出名字翻译版本不同但是是同一个人也认为正确。
    - 例如，如果标准答案是"Robinson"，那么回答鲁滨逊或者鲁滨孙均正确。

下面是一个新的问题示例。请只回复A、B、C之一，不要道歉或纠正自己的错误，只需要评估该回答。
```
问题: {question}
正确答案: {target}
预测答案: {predicted_answer}
```

将此新问题的预测答案评定为以下之一：
A:【正确】
B:【错误】
C:【未尝试】

只返回字母"A"、"B"或"C"，无须添加其他文本。
""".strip()

SUBSET_LIST = ['中华文化', '人文与社会科学', '工程、技术与应用科学', '生活、艺术与文化', '社会', '自然与自然科学']


@register_benchmark(
    BenchmarkMeta(
        name='chinese_simpleqa',
        pretty_name='Chinese-SimpleQA',
        tags=[Tags.KNOWLEDGE, Tags.QA, Tags.CHINESE],
        description=
        "Chinese SimpleQA is a Chinese question-answering dataset designed to evaluate the performance of language models on simple factual questions. It includes a variety of topics and is structured to test the model's ability to understand and generate correct answers in Chinese.",  # noqa: E501
        subset_list=SUBSET_LIST,
        dataset_id='AI-ModelScope/Chinese-SimpleQA',
        metric_list=['is_correct', 'is_incorrect', 'is_not_attempted'],
        few_shot_num=0,
        train_split=None,
        eval_split='train',
        prompt_template='请回答问题：\n\n{question}'
    )
)
class ChineseSimpleQAAdapter(DefaultDataAdapter):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._use_llm_judge = True  # Use LLM as a judge by default
        self.reformat_subset = True  # Reformat subset to primary_category

    def record_to_sample(self, record: Dict[str, Any]) -> Sample:
        """
        Convert a data record to a Sample object.

        Args:
            record (Dict[str, Any]): Input data record.

        Returns:
            Sample: Sample object with input, target, and metadata.
        """
        question = record['question']
        answer = record['answer']
        subset_key = record.get('primary_category', 'default')  # Use primary_category as subset key
        metadata = {
            'id': record.get('id', 'unknown'),
            'primary_category': subset_key,
            'secondary_category': record.get('secondary_category', '')
        }

        return Sample(input=question, target=answer, subset_key=subset_key, metadata=metadata)

    def llm_match_score(
        self,
        original_prediction: str,
        filtered_prediction: str,
        reference: str,
        task_state: TaskState,
    ) -> Score:
        score = Score(
            extracted_prediction=filtered_prediction,
            prediction=original_prediction,
        )

        question = task_state.input_text

        # Request judge and obtain score
        prompt = GRADER_TEMPLATE.format(question=question, target=reference, predicted_answer=filtered_prediction)
        system_prompt = '你是一个智能助手，请根据给定问题、标准答案和模型预测的答案来评估模型的回答是否正确。'
        judge_response = self.llm_judge.judge(prompt, system_prompt=system_prompt)
        # parse grading response
        match = re.search(r'(A|B|C)', judge_response)
        res = match.group(0) if match else 'C'

        # Set score based on the match result
        score.value = {
            'is_correct': 1 if res == 'A' else 0,
            'is_incorrect': 1 if res == 'B' else 0,
            'is_not_attempted': 1 if res == 'C' else 0,
        }
        score.explanation = f'LLM judge: {judge_response}'
        score.metadata = {
            'source': 'llm_judge',
            'judge_strategy': self.judge_strategy,
            'model': self.llm_judge.model_id
        }
        score.main_score_name = 'is_correct'
        return score



# adapted from https://github.com/THUDM/ImageReward/tree/main/ImageReward

import os
import torch
import torch.nn as nn
from PIL import Image
from torchvision.transforms import CenterCrop, Compose, Normalize, Resize, ToTensor

from .blip_pretrain import BLIP_Pretrain

try:
    from torchvision.transforms import InterpolationMode
    BICUBIC = InterpolationMode.BICUBIC
except ImportError:
    BICUBIC = Image.BICUBIC


def _convert_image_to_rgb(image):
    return image.convert('RGB')


def _transform(n_px):
    return Compose([
        Resize(n_px, interpolation=BICUBIC),
        CenterCrop(n_px),
        _convert_image_to_rgb,
        ToTensor(),
        Normalize((0.48145466, 0.4578275, 0.40821073), (0.26862954, 0.26130258, 0.27577711)),
    ])


class MLP(nn.Module):

    def __init__(self, input_size):
        super().__init__()
        self.input_size = input_size

        self.layers = nn.Sequential(
            nn.Linear(self.input_size, 1024),
            #nn.ReLU(),
            nn.Dropout(0.2),
            nn.Linear(1024, 128),
            #nn.ReLU(),
            nn.Dropout(0.2),
            nn.Linear(128, 64),
            #nn.ReLU(),
            nn.Dropout(0.1),
            nn.Linear(64, 16),
            #nn.ReLU(),
            nn.Linear(16, 1)
        )

        # initial MLP param
        for name, param in self.layers.named_parameters():
            if 'weight' in name:
                nn.init.normal_(param, mean=0.0, std=1.0 / (self.input_size + 1))
            if 'bias' in name:
                nn.init.constant_(param, val=0)

    def forward(self, input):
        return self.layers(input)


class ImageReward(nn.Module):

    def __init__(self, med_config, device='cpu'):
        super().__init__()
        self.device = device

        self.blip = BLIP_Pretrain(image_size=224, vit='large', med_config=med_config)
        self.preprocess = _transform(224)
        self.mlp = MLP(768)

        self.mean = 0.16717362830052426
        self.std = 1.0333394966054072

    def score_gard(self, prompt_ids, prompt_attention_mask, image):

        image_embeds = self.blip.visual_encoder(image)
        # text encode cross attention with image
        image_atts = torch.ones(image_embeds.size()[:-1], dtype=torch.long).to(self.device)
        text_output = self.blip.text_encoder(
            prompt_ids,
            attention_mask=prompt_attention_mask,
            encoder_hidden_states=image_embeds,
            encoder_attention_mask=image_atts,
            return_dict=True,
        )

        txt_features = text_output.last_hidden_state[:, 0, :]  # (feature_dim)
        rewards = self.mlp(txt_features)
        rewards = (rewards - self.mean) / self.std

        return rewards

    def score(self, prompt, image):

        if (type(image).__name__ == 'list'):
            _, rewards = self.inference_rank(prompt, image)
            return rewards

        # text encode
        text_input = self.blip.tokenizer(
            prompt, padding='max_length', truncation=True, max_length=35, return_tensors='pt'
        ).to(self.device)

        # image encode
        if isinstance(image, Image.Image):
            pil_image = image
        elif isinstance(image, str) and os.path.isfile(image):
            pil_image = Image.open(image)
        else:
            raise TypeError(
                r'This image parameter type has not been supportted yet. Please pass PIL.Image or file path str.'
            )

        image = self.preprocess(pil_image).unsqueeze(0).to(self.device)
        image_embeds = self.blip.visual_encoder(image)

        # text encode cross attention with image
        image_atts = torch.ones(image_embeds.size()[:-1], dtype=torch.long).to(self.device)
        text_output = self.blip.text_encoder(
            text_input.input_ids,
            attention_mask=text_input.attention_mask,
            encoder_hidden_states=image_embeds,
            encoder_attention_mask=image_atts,
            return_dict=True,
        )

        txt_features = text_output.last_hidden_state[:, 0, :].float()  # (feature_dim)
        rewards = self.mlp(txt_features)
        rewards = (rewards - self.mean) / self.std

        return rewards.detach().cpu().numpy().item()

    def inference_rank(self, prompt, generations_list):

        text_input = self.blip.tokenizer(
            prompt, padding='max_length', truncation=True, max_length=35, return_tensors='pt'
        ).to(self.device)

        txt_set = []
        for generation in generations_list:
            # image encode
            if isinstance(generation, Image.Image):
                pil_image = generation
            elif isinstance(generation, str):
                if os.path.isfile(generation):
                    pil_image = Image.open(generation)
            else:
                raise TypeError(
                    r'This image parameter type has not been supportted yet. Please pass PIL.Image or file path str.'
                )
            image = self.preprocess(pil_image).unsqueeze(0).to(self.device)
            image_embeds = self.blip.visual_encoder(image)

            # text encode cross attention with image
            image_atts = torch.ones(image_embeds.size()[:-1], dtype=torch.long).to(self.device)
            text_output = self.blip.text_encoder(
                text_input.input_ids,
                attention_mask=text_input.attention_mask,
                encoder_hidden_states=image_embeds,
                encoder_attention_mask=image_atts,
                return_dict=True,
            )
            txt_set.append(text_output.last_hidden_state[:, 0, :])

        txt_features = torch.cat(txt_set, 0).float()  # [image_num, feature_dim]
        rewards = self.mlp(txt_features)  # [image_num, 1]
        rewards = (rewards - self.mean) / self.std
        rewards = torch.squeeze(rewards)
        _, rank = torch.sort(rewards, dim=0, descending=True)
        _, indices = torch.sort(rank, dim=0)
        indices = indices + 1

        return indices.detach().cpu().numpy().tolist(), rewards.detach().cpu().numpy().tolist()
